/**********************************************************************
 "Copyright 1990-2018 Brian MacWhinney. Use is subject to Gnu Public License
 as stated in the attached "gpl.txt" file."
*/

//  kideval -c1 '+t*CHI'
// /WORK/clanW/src/unix/bin/kideval -c1 '+t*CHI' -L/WORK/clanW/src/unix/bin/kideval_lib
// /WORK/clanW/src/unix/bin/kideval '+t*CHI' -re -leng -L/WORK/clanW/src/unix/bin/kideval_lib *.cha

/*
 isCreateDB

 @ID   = +
 # gem results and unique words
 @G    = G
 @BG   = B
 @EG   = E
 @END  = -

 kideval_initTSVars
 prTSResults - write DB
 retrieveTS  - read DB
 wID
	wnum;
	wused;
 addFilePath

 KIDEVAL_DB_ID
 result = DoKidevalDB(theItem);
 KidevalDBMenu
 KIDEVAL_DB_item
*/

//#define DEBUGEVALRESULTS
//#define GEMS_COUNT

#define KIDEVAL_DB_VERSION 3

#define NUMGEMITEMS 10
#define DATABASE_FILE_NAME "_kideval_db.cut"
#define DATABSEFILESLIST "0kideval_Database_IDs.cex"
#define DATABASEFILESDEBUG "0kideval_DB_results.txt"


#define CHAT_MODE 1
#include "cu.h"
#include "c_curses.h"
#include "dss.h"
#include "ipsyn.h"
#include "vocd/vocdefs.h"
#include "vocd/speaker.h"
#if !defined(_MAC_CODE)
	#include <sys/types.h>
#endif
#include <time.h>
#include <math.h>
#if defined(_MAC_CODE) || defined(UNX)
	#include <sys/stat.h>
	#include <dirent.h>
#endif

#if !defined(UNX)
#define _main kideval_main
#define call kideval_call
#define getflag kideval_getflag
#define init kideval_init
#define usage kideval_usage
#endif

#define IS_WIN_MODE FALSE
#include "mul.h" 

extern char OverWriteFile;
extern char outputOnlyData;
extern char isRecursive;
extern char isLanguageExplicit;
extern struct tier *defheadtier;
extern struct IDtype *IDField;
extern struct tier *headtier;
extern FNType prefsDir[];

#define SDRESSIZE 256
#define NUMCOMPS 128

enum {
	BASE_COM = 0,
	DSS_COM,
	VOCD_COM,
	IPSYN_COM,
	LAST_COMMAND
} ;

struct label_cols {
	char *label;
	int  num;
	struct label_cols *next_label;
};

#define COLS struct all_cols
COLS {
	MORWDLST *pats;
	struct label_cols *labelP;
	char isClitic;
	COLS *next_col;
};

struct kideval_words {
	char *word;
	int  wnum;
	char wused;
	struct kideval_words *left;
	struct kideval_words *right;
};

struct kideval_speakers {
	char isSpeakerFound;
	char isMORFound;
	char isPSDFound; // +/.
	char *fname;
	char *sp;
	char *ID;
	int  wID;
	float tUtt;
	float frTokens;	// total number of words
	float frTypes;	// number of diff words
	float NDW, NDWTotal;	// number of diff words
	float CUR;
	float mWords, morf, mUtt;
	float mWords100, morf100, mUtt100;
	float tdWords, tdUtts, tdur;
	float werr, uerr;
	float retr, repet;
	DSSSP *dss_sp;
	float dssUttCnt;
	float dssGTotal;
	VOCDSP *vocd_sp;
	float vocdOptimum;
	IPSYN_SP *ipsyn_sp;
	float ipsynUttCnt;
	float ipsynScore;
	float morTotal;	// total number of words on %mor tier
	struct kideval_words *words_root;
	struct kideval_words *NDW_root;
	float *mor_count;
	struct kideval_speakers *next_sp;
} ;

struct SDs {
	char isSDComputed;
	float dbSD;
	char stars;
} ;

struct database {
	float tUtt_sqr, tUtt, mn_tUtt, mx_tUtt, tUtt_num;
	float frTokens_sqr, frTokens, mn_frTokens, mx_frTokens, frTokens_num;	/* total number of words */
	float frTypes_sqr, frTypes, mn_frTypes, mx_frTypes, frTypes_num;	/* number of diff words */
	float NDW_sqr, NDW, mn_NDW, mx_NDW, NDW_num;	/* number of diff words */
	float TTR_sqr, TTR, mn_TTR, mx_TTR, TTR_num;
	float CUR_sqr, CUR, mn_CUR, mx_CUR, CUR_num;
	float mUtt_sqr, mUtt, mn_mUtt, mx_mUtt, mUtt_num;
	float mWords_sqr, mWords, mn_mWords, mx_mWords, mWords_num;
	float morf_sqr, morf, mn_morf, mx_morf, morf_num;
	float mUtt100_sqr, mUtt100, mn_mUtt100, mx_mUtt100, mUtt100_num;
	float mWords100_sqr, mWords100, mn_mWords100, mx_mWords100, mWords100_num;
	float morf100_sqr, morf100, mn_morf100, mx_morf100, morf100_num;
	float tdWords_sqr, tdWords, mn_tdWords, mx_tdWords, tdWords_num;
	float tdUtts_sqr, tdUtts, mn_tdUtts, mx_tdUtts, tdUtts_num;
	float tdur_sqr, tdur, mn_tdur, mx_tdur, tdur_num;
	float tdWT_sqr, tdWT, mn_tdWT, mx_tdWT, tdWT_num;
	float tdUT_sqr, tdUT, mn_tdUT, mx_tdUT, tdUT_num;
	float werr_sqr, werr, mn_werr, mx_werr, werr_num;
	float uerr_sqr, uerr, mn_uerr, mx_uerr, uerr_num;
	float retr_sqr, retr, mn_retr, mx_retr, retr_num;
	float repet_sqr, repet, mn_repet, mx_repet, repet_num;
	float dssUttCnt_sqr, dssUttCnt, mn_dssUttCnt, mx_dssUttCnt, dssUttCnt_num;
	float dssGTotal_sqr, dssGTotal, mn_dssGTotal, mx_dssGTotal, dssGTotal_num;
	float vocdOptimum_sqr, vocdOptimum, mn_vocdOptimum, mx_vocdOptimum, vocdOptimum_num;
	float ipsynUttCnt_sqr, ipsynUttCnt, mn_ipsynUttCnt, mx_ipsynUttCnt, ipsynUttCnt_num;
	float ipsynScore_sqr, ipsynScore, mn_ipsynScore, mx_ipsynScore, ipsynScore_num;
	float morTotal_sqr, morTotal, mn_morTotal, mx_morTotal, morTotal_num;	/* total number of words on %mor tier */
	float *morItems_sqr, *morItems, morItems_num;
	struct kideval_speakers *db_sp;
} ;

struct DBKeys {
	char *key1;
	char *key2;
	char *key3;
	char *key4;
	char *age;
	int  agef;
	int  aget;
	char *sex;
	struct DBKeys *next;
};

static int  targc;
#if defined(_MAC_CODE) || defined(_WIN32)
static char **targv;
#endif
#ifdef UNX
static char *targv[MAX_ARGS];
#endif
static int  kideval_SpecWords, DBGemNum, MinUttLimit, DssIpsynUttLimit, wdOffset;
static int  colLabelsNum, CRDBcnt;
static int  gMisalignmentError;
static char *DBGems[NUMGEMITEMS], *script_file;
static char dssRulesFName[256], DB_version[65], DB_type[128];
static char ftime, isDBFilesList, isCreateDB, isRawVal, isLinkAge;
static char kideval_BBS[5], kideval_CBS[5], kideval_group, kideval_n_option, isNOptionSet, onlyApplyToDB, isGOptionSet, GemMode;
static FILE	*dbfpout;
static FILE *DBFilesListFP;
static struct DBKeys *DBKeyRoot;
static struct kideval_speakers *sp_head;
static struct database *gdb;
static COLS *colsRoot;
static struct label_cols *labelsRoot;
#ifdef DEBUGEVALRESULTS
FILE *dbResults;
#endif

void usage() {
	printf("Usage: kideval [aN bS dS eN g gS lF n oN s %s] filename(s)\n",mainflgs());
	puts("+aN: limit analysis to files containing more than N utterances (default: no minimal limit)");
	printf("+bS: add all S characters to morpheme delimiters list (default: %s)\n", rootmorf);
	puts("-bS: remove all S characters from be morphemes list (-b: empty morphemes list)");
#if defined(_CLAN_DEBUG)
	puts("+c : create KIDEVAL database file (kideval +leng +c_fp)");
	puts("    Set \"working\" to: ~/SERVERS/data");
	puts("+c : create KIDEVAL database file (kideval +lzho +c_md)");
	puts("    Set \"working\" to: ~/SERVERS/data");
	puts("+c : create KIDEVAL database file (kideval +ljpn +c_td)");
	puts("    Set \"working\" to: ~/SERVERS/data");
#endif // _CLAN_DEBUG
	puts("+dfp~S: specify database keyword(s) S. For example:");
	puts("    +dfp~\"2;7-3;\" for children of age 31 to 36 months old.");
	puts("    +dfp~\"2;-2;6|female\" for females children of age 24 to 30 months old.");
	puts("+dmd~S: specify database keyword(s) S. For example:");
	puts("    +dmd~\"2;7-3;\" for children of age 31 to 36 months old.");
	puts("    +dmd~\"2;-2;6|female\" for females children of age 24 to 30 months old.");
	puts("+e1: create list of database files used for comparisons");
#if defined(_CLAN_DEBUG) || defined(UNX)
	puts("+g : Gem tier should contain all words specified by +gS");
	puts("-g : look for Gems in Database only");
	puts("+gS: select Gems which are labeled by label S");
#endif
	puts("+lF: specify language script file name F (default: eng)");
#ifdef UNX
	puts("+LF: specify full path F of the lib folder");
#endif
#if defined(_CLAN_DEBUG) || defined(UNX)
	puts("+n : Gem is terminated by the next @G (default: automatic detection)");
	puts("-n : Gem is defined by @BG and @EG (default: automatic detection)");
#endif
	puts("+o4: output percentage values instead of raw values");
	puts("+qN: change the minimum limit of utterance for DSS and IPSYN (default: DSS=50, IPSYN=100)");
	puts("+s : counts utterances and other words found along with \"xxx\"");
	mainusage(FALSE);
#ifdef UNX
	printf("CURRENT DATABASE COMPARISONS ARE PRELIMINARY AND SHOULD NOT BE USED FOR CLINICAL DIAGNOSIS.\n");
	printf("PLEASE SEE SLP MANUAL FOR RECOMMENDED INTERPRETATION.\n\n");
#else
	printf("%c%cCURRENT DATABASE COMPARISONS ARE PRELIMINARY AND SHOULD NOT BE USED FOR CLINICAL DIAGNOSIS.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
	printf("%c%cPLEASE SEE SLP MANUAL FOR RECOMMENDED INTERPRETATION.%c%c\n\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	puts("IF NO \"+t*...\" option specified, then speaker defaults to \"*CHI:\"");
	puts("IF NO \"+l*...\" option specified, then language defaults to \"eng\"");
	puts("Examples:");
	puts("   Search database for female children between ages of 24 and 30 months");
	puts("       kideval +dfp~\"2;-2;6|female\" *.cha");
	puts("       kideval -lzho +dmd~\"2;-2;6|female\" *.cha");
	puts("   Search database for children between ages of 31 and 36 months");
	puts("       kideval +dfp~\"2;7-3;\" *.cha ");
	puts("       kideval -lzho +dmd~\"2;7-3;\" *.cha ");
	puts("   Compute results for \"Breakfast\" Gems of English speaker *CHI");
	puts("       kideval +gBreakfast *.cha");
	puts("   Compute results only for English speaker *CHI");
	puts("       kideval *.cha");
	puts("   Compute results only for Spanish speaker *PAR");
	puts("       kideval +t*par +lspa *.cha");
	cutt_exit(0);
}

static void init_db(struct database *p) {
	int i;

	p->tUtt_sqr = 0.0; p->tUtt = 0.0; p->mn_tUtt = 0.0; p->mx_tUtt = 0.0; p->tUtt_num = 0.0;
	p->frTokens_sqr = 0.0; p->frTokens = 0.0; p->mn_frTokens = 0.0; p->mx_frTokens = 0.0; p->frTokens_num = 0.0;
	p->frTypes_sqr = 0.0; p->frTypes = 0.0; p->mn_frTypes = 0.0; p->mx_frTypes = 0.0; p->frTypes_num = 0.0;
	p->NDW_sqr = 0.0; p->NDW = 0.0; p->mn_NDW = 0.0; p->mx_NDW = 0.0; p->NDW_num = 0.0;
	p->TTR_sqr = 0.0; p->TTR = 0.0; p->mn_TTR = 0.0; p->mx_TTR = 0.0; p->TTR_num = 0.0;
	p->CUR_sqr = 0.0; p->CUR = 0.0; p->mn_CUR = 0.0; p->mx_CUR = 0.0; p->CUR_num = 0.0;
	p->mUtt_sqr = 0.0; p->mUtt = 0.0; p->mn_mUtt = 0.0; p->mx_mUtt = 0.0; p->mUtt_num = 0.0;
	p->mWords_sqr = 0.0; p->mWords = 0.0; p->mn_mWords = 0.0; p->mx_mWords = 0.0; p->mWords_num = 0.0;
	p->morf_sqr = 0.0; p->morf = 0.0; p->mn_morf = 0.0; p->mx_morf = 0.0; p->morf_num = 0.0;
	p->mUtt100_sqr = 0.0; p->mUtt100 = 0.0; p->mn_mUtt100 = 0.0; p->mx_mUtt100 = 0.0; p->mUtt100_num = 0.0;
	p->mWords100_sqr = 0.0; p->mWords100 = 0.0; p->mn_mWords100 = 0.0; p->mx_mWords100 = 0.0;  p->mWords100_num = 0.0;
	p->morf100_sqr = 0.0; p->morf100 = 0.0; p->mn_morf100 = 0.0; p->mx_morf100 = 0.0; p->morf100_num = 0.0;
	p->tdWords_sqr = 0.0; p->tdWords = 0.0; p->mn_tdWords = 0.0; p->mx_tdWords = 0.0; p->tdWords_num = 0.0;
	p->tdUtts_sqr = 0.0; p->tdUtts = 0.0; p->mn_tdUtts = 0.0; p->mx_tdUtts = 0.0; p->tdUtts_num = 0.0;
	p->tdur_sqr = 0.0; p->tdur = 0.0; p->mn_tdur = 0.0; p->mx_tdur = 0.0; p->tdur_num = 0.0;
	p->tdWT_sqr = 0.0; p->tdWT = 0.0; p->mn_tdWT = 0.0; p->mx_tdWT = 0.0; p->tdWT_num = 0.0;
	p->tdUT_sqr = 0.0; p->tdUT = 0.0; p->mn_tdUT = 0.0; p->mx_tdUT = 0.0; p->tdUT_num = 0.0;
	p->werr_sqr = 0.0; p->werr = 0.0; p->mn_werr = 0.0; p->mx_werr = 0.0; p->werr_num = 0.0;
	p->uerr_sqr = 0.0; p->uerr = 0.0; p->mn_uerr = 0.0; p->mx_uerr = 0.0; p->uerr_num = 0.0;
	p->retr_sqr = 0.0; p->retr = 0.0; p->mn_retr = 0.0; p->mx_retr = 0.0; p->retr_num = 0.0;
	p->repet_sqr = 0.0; p->repet = 0.0; p->mn_repet = 0.0; p->mx_repet = 0.0; p->repet_num = 0.0;
	p->dssGTotal_sqr = 0.0; p->dssGTotal = 0.0; p->mn_dssGTotal = 0.0; p->mx_dssGTotal = 0.0; p->dssGTotal_num = 0.0;
	p->dssUttCnt_sqr = 0.0; p->dssUttCnt = 0.0; p->mn_dssUttCnt = 0.0; p->mx_dssUttCnt = 0.0; p->dssUttCnt_num = 0.0;
	p->vocdOptimum_sqr = 0.0; p->vocdOptimum = 0.0; p->mn_vocdOptimum = 0.0; p->mx_vocdOptimum = 0.0; p->vocdOptimum_num = 0.0;
	p->ipsynUttCnt_sqr = 0.0; p->ipsynUttCnt = 0.0; p->mn_ipsynUttCnt = 0.0; p->mx_ipsynUttCnt = 0.0; p->ipsynUttCnt_num = 0.0;
	p->ipsynScore_sqr = 0.0; p->ipsynScore = 0.0; p->mn_ipsynScore = 0.0; p->mx_ipsynScore = 0.0; p->ipsynScore_num = 0.0;
	p->morTotal_sqr = 0.0; p->morTotal = 0.0; p->mn_morTotal = 0.0; p->mx_morTotal = 0.0; p->morTotal_num = 0.0;

	for (i=0; i < colLabelsNum; i++) {
		p->morItems_sqr[i] = 0.0; p->morItems[i] = 0.0;
	}
	gdb->morItems_num = 0.0;

	p->db_sp = NULL;
}

static struct label_cols *freeLabels(struct label_cols *p) {
	struct label_cols *t;

	while (p != NULL) {
		t = p;
		p = p->next_label;
		if (t->label)
			free(t->label);
		free(t);
	}
	return(NULL);
}

static COLS *freeColsP(COLS *p) {
	COLS *t;

	while (p != NULL) {
		t = p;
		p = p->next_col;
		t->pats = freeMorWords(t->pats);
		free(t);
	}
	return(NULL);
}

static struct kideval_words *kideval_freetree(struct kideval_words *p) {
	if (p != NULL) {
		kideval_freetree(p->left);
		kideval_freetree(p->right);
		if (p->word != NULL)
			free(p->word);
		free(p);
	}
	return(NULL);
}

static struct kideval_speakers *freespeakers(struct kideval_speakers *p) {
	struct kideval_speakers *ts;

	while (p) {
		ts = p;
		p = p->next_sp;
		if (ts->fname != NULL)
			free(ts->fname);
		if (ts->sp != NULL)
			free(ts->sp);
		if (ts->ID != NULL)
			free(ts->ID);
		if (ts->words_root != NULL)
			kideval_freetree(ts->words_root);
		if (ts->NDW_root != NULL)
			kideval_freetree(ts->NDW_root);
		if (ts->mor_count != NULL)
			free(ts->mor_count);
		free(ts);
	}
	return(NULL);
}

static struct database *freedb(struct database *db) {
	if (db != NULL) {
		db->db_sp = freespeakers(db->db_sp);
		if (db->morItems_sqr != NULL)
			free(db->morItems_sqr);
		if (db->morItems != NULL)
			free(db->morItems);
		free(db);
	}
	return(NULL);
}

static struct DBKeys *freeDBKeys(struct DBKeys *p) {
	struct DBKeys *t;

	while (p) {
		t = p;
		p = p->next;
		if (t->key1 != NULL)
			free(t->key1);
		if (t->key2 != NULL)
			free(t->key2);
		if (t->key3 != NULL)
			free(t->key3);
		if (t->key4 != NULL)
			free(t->key4);
		if (t->age != NULL)
			free(t->age);
		if (t->sex != NULL)
			free(t->sex);
		free(t);
	}
	return(NULL);
}

static void kideval_error(struct database *tdb, char IsOutOfMem) {
	if (IsOutOfMem)
		fputs("ERROR: Out of memory.\n",stderr);
	sp_head = freespeakers(sp_head);
	DBKeyRoot = freeDBKeys(DBKeyRoot);
	labelsRoot = freeLabels(labelsRoot);
	colsRoot = freeColsP(colsRoot);
	if (tdb != NULL)
		tdb = freedb(tdb);
	else
		gdb = freedb(gdb);
	dss_freeSpeakers();
	dss_cleanSearchMem();
	ipsyn_freeSpeakers();
	ipsyn_cleanSearchMem();
	speakers = speaker_free_up_speakers(speakers, TRUE);
	if (VOCD_TYPE_D)
		denom_speakers = speaker_free_up_speakers(denom_speakers, TRUE);
	cutt_exit(0);
}

static void addDBGems(char *gem) {
	if (DBGemNum >= NUMGEMITEMS) {
		fprintf(stderr, "\nERROR: Too many keywords specified. The limit is %d\n", NUMGEMITEMS);
		kideval_error(NULL, FALSE);
	}
	DBGems[DBGemNum] = gem;
	DBGemNum++;
}

static int excludeGemKeywords(char *word) {
	int i;

	if (word[0] == '+' || strcmp(word, "!") == 0 || strcmp(word, "?") == 0 || strcmp(word, ".") == 0)
		return(FALSE);
	for (i=0; i < DBGemNum; i++) {
		if (uS.mStricmp(DBGems[i], word) == 0)
			return(TRUE);
	}
	return(FALSE);
}

static char *kideval_strsave(const char *s) {
	char *p;
	
	if ((p=(char *)malloc(strlen(s)+1)) != NULL)
		strcpy(p, s);
	else
		kideval_error(NULL, TRUE);
	return(p);
}

static struct DBKeys *initDBKey(const char *k1, char *k2, char *k3, char *k4, char *a, int af, int at, char *s) {
	struct DBKeys *p;

	p = NEW(struct DBKeys);
	if (p == NULL)
		kideval_error(NULL, TRUE);
	if (k1 == NULL)
		p->key1 = NULL;
	else
		p->key1 = kideval_strsave(k1);
	if (k2 == NULL)
		p->key2 = NULL;
	else
		p->key2 = kideval_strsave(k2);
	if (k3 == NULL)
		p->key3 = NULL;
	else
		p->key3 = kideval_strsave(k3);
	if (k4 == NULL)
		p->key4 = NULL;
	else
		p->key4 = kideval_strsave(k4);
	if (a == NULL)
		p->age = NULL;
	else
		p->age = kideval_strsave(a);
	p->agef = af;
	p->aget = at;
	if (s == NULL)
		p->sex = NULL;
	else
		p->sex = kideval_strsave(s);
	p->next = NULL;
	return(p);
}

static void addDBKeys(const char *key) {
	char *b, *e, isFound;
	struct DBKeys *DBKey, *cDBKey;

	DBKey = initDBKey(NULL, NULL, NULL, NULL, NULL, 0, 0, NULL);
	isFound = FALSE;
	strcpy(templineC4, key);
	b = templineC4;
	do {
		e = strchr(b, '|');
		if (e != NULL) {
			*e = EOS;
			e++;
		}
		uS.remFrontAndBackBlanks(b);
		if (*b != EOS) {
			if (uS.mStricmp(b,"male")==0 || uS.mStricmp(b,"female")==0) {
				DBKey->sex = kideval_strsave(b);
			} else if (isAge(b, &DBKey->agef, &DBKey->aget)) {
				if (DBKey->aget == 0) {
					fprintf(stderr, "\nERROR: Please specify the age range instead of just \"%s\"\n", b);
					fprintf(stderr, "For example: \"60-80\" means people between 60 and 80 years old.\n");
					kideval_error(NULL, FALSE);
				}
				DBKey->age = kideval_strsave(b);
			} else {
				if (DBKey->key1 == NULL)
					DBKey->key1 = kideval_strsave(b);
				else if (DBKey->key2 == NULL)
					DBKey->key2 = kideval_strsave(b);
				else if (DBKey->key3 == NULL)
					DBKey->key3 = kideval_strsave(b);
				else if (DBKey->key4 == NULL)
					DBKey->key4 = kideval_strsave(b);
			}
			isFound = TRUE;
		}
		b = e;
	} while (b != NULL) ;
	if (isFound) {
		if (DBKeyRoot == NULL) {
			DBKeyRoot = DBKey;
		} else {
			for (cDBKey=DBKeyRoot; cDBKey->next != NULL; cDBKey=cDBKey->next) ;
			cDBKey->next = DBKey;
		}
	}
}

static int isEqualKey(char *DBk, char *IDk) {
	if (DBk == NULL || IDk == NULL) {
		return(1);
	}
	return(uS.mStricmp(IDk, DBk));
}

static void changeAgeField(char *fr, char *to) {
	int i, j, k, cnt, years, months, days;
	char age[BUFSIZ], *s;

	j = 0;
	cnt = 0;
	for (i=0; cnt < 3 && fr[i] != EOS; i++) {
		to[j++] = fr[i];
		if (fr[i] == '|')
			cnt++;
	}
	if (cnt == 3 && fr[i] != EOS) {
		k = 0;
		for (; fr[i] != '|' && fr[i] != EOS; i++) {
			age[k++] = fr[i];
		}
		age[k] = EOS;
		s = strchr(age, ';');
		if (s != NULL) {
			years = atoi(age);
			strcpy(age, s+1);
		} else
			years = 0;
		s = strchr(age, '.');
		if (s != NULL) {
			months = atoi(age);
			strcpy(age, s+1);
			days = atoi(age);
		} else {
			months = 0;
			days = 0;
		}
		cnt = years * 12;
		cnt = cnt + months;
		if (days >= 15)
			cnt++;
		sprintf(to+j, "%d", cnt);
		k = j;
		j = j + strlen(to+k);
	}
	for (; fr[i] != EOS; i++)
		to[j++] = fr[i];
	to[j] = EOS;
}

static char isKeyMatch(struct IDparts *IDt) {
	int  cAgef, cAget, numSpec, numMatched;
	struct DBKeys *DBKey;

	for (DBKey=DBKeyRoot; DBKey != NULL; DBKey=DBKey->next) {
		numSpec = 0;
		numMatched = 0;
		if (DBKey->key1 != NULL) {
			numSpec++;
			if (isEqualKey(DBKey->key1, IDt->lang) == 0 || isEqualKey(DBKey->key1, IDt->corp) == 0  ||
				isEqualKey(DBKey->key1, IDt->code) == 0 || isEqualKey(DBKey->key1, IDt->group) == 0 ||
				isEqualKey(DBKey->key1, IDt->SES) == 0  || isEqualKey(DBKey->key1, IDt->role) == 0  ||
				isEqualKey(DBKey->key1, IDt->edu) == 0  || isEqualKey(DBKey->key1, IDt->UNQ) == 0)
				numMatched++;
		}
		if (DBKey->key2 != NULL) {
			numSpec++;
			if (isEqualKey(DBKey->key2, IDt->lang) == 0 || isEqualKey(DBKey->key2, IDt->corp) == 0  ||
				isEqualKey(DBKey->key2, IDt->code) == 0 || isEqualKey(DBKey->key2, IDt->group) == 0 ||
				isEqualKey(DBKey->key2, IDt->SES) == 0  || isEqualKey(DBKey->key2, IDt->role) == 0  ||
				isEqualKey(DBKey->key2, IDt->edu) == 0  || isEqualKey(DBKey->key2, IDt->UNQ) == 0)
				numMatched++;
		}
		if (DBKey->key3 != NULL) {
			numSpec++;
			if (isEqualKey(DBKey->key3, IDt->lang) == 0 || isEqualKey(DBKey->key3, IDt->corp) == 0  ||
				isEqualKey(DBKey->key3, IDt->code) == 0 || isEqualKey(DBKey->key3, IDt->group) == 0 ||
				isEqualKey(DBKey->key3, IDt->SES) == 0  || isEqualKey(DBKey->key3, IDt->role) == 0  ||
				isEqualKey(DBKey->key3, IDt->edu) == 0  || isEqualKey(DBKey->key3, IDt->UNQ) == 0)
				numMatched++;
		}
		if (DBKey->key4 != NULL) {
			numSpec++;
			if (isEqualKey(DBKey->key4, IDt->lang) == 0 || isEqualKey(DBKey->key4, IDt->corp) == 0  ||
				isEqualKey(DBKey->key4, IDt->code) == 0 || isEqualKey(DBKey->key4, IDt->group) == 0 ||
				isEqualKey(DBKey->key4, IDt->SES) == 0  || isEqualKey(DBKey->key4, IDt->role) == 0  ||
				isEqualKey(DBKey->key4, IDt->edu) == 0  || isEqualKey(DBKey->key4, IDt->UNQ) == 0)
				numMatched++;
		}
		if (DBKey->age != NULL) {
			numSpec++;
			isAge(IDt->ages, &cAgef, &cAget);
			if (cAget != 0) {
				if ((DBKey->agef <= cAgef && cAgef <= DBKey->aget) ||
					(DBKey->agef <= cAget && cAget <= DBKey->aget))
					numMatched++;
			} else {
				if (DBKey->agef <= cAgef && cAgef <= DBKey->aget)
					numMatched++;
			}
		}
		if (DBKey->sex != NULL) {
			numSpec++;
			if (isEqualKey(DBKey->sex, IDt->sex) == 0)
				numMatched++;
		}
		if (numSpec == numMatched)
			return(TRUE);
	}
	return(FALSE);
}

static void kideval_resettree(struct kideval_words *p) {
	if (p != NULL) {
		kideval_resettree(p->left);
		p->wused = FALSE;
		kideval_resettree(p->right);
	}
}

static void kideval_initTSVars(struct kideval_speakers *ts, char isAll) {
	int i;

	ts->isMORFound = FALSE;
	ts->isPSDFound = FALSE;
	ts->tUtt	= 0.0;
	ts->frTokens= 0.0;
	ts->frTypes = 0.0;
	ts->NDW		= 0.0;
	ts->NDWTotal= 0.0;
	ts->vocd_sp  = NULL;
	ts->vocdOptimum = 0.0;
	ts->CUR		= 0.0;
	ts->mWords  = 0.0;
	ts->morf	= 0.0;
	ts->mUtt	= 0.0;
	ts->mWords100= 0.0;
	ts->morf100	= 0.0;
	ts->mUtt100	= 0.0;
	ts->tdWords	= 0.0;
	ts->tdUtts	= 0.0;
	ts->tdur	= 0.0;
	ts->werr    = 0.0;
	ts->uerr    = 0.0;
	ts->retr    = 0.0;
	ts->repet   = 0.0;
	ts->dss_sp  = NULL;
	ts->dssUttCnt= 0.0;
	ts->dssGTotal= 0.0;
	ts->ipsyn_sp = NULL;
	ts->ipsynUttCnt = 0.0;
	ts->ipsynScore = 0.0;
	ts->morTotal   = 0.0;
	if (colLabelsNum > 0) {
		ts->mor_count = (float *)malloc(sizeof(float) * colLabelsNum);
		if (ts->mor_count == NULL) {
			kideval_error(gdb, TRUE);
		}
		for (i=0; i < colLabelsNum; i++)
			ts->mor_count[i] = 0.0;
	} else
		ts->mor_count = NULL;
	if (isAll) {
		ts->wID = 0;
		ts->words_root = NULL;
		ts->NDW_root   = NULL;
	} else {
		kideval_resettree(ts->words_root);
		kideval_resettree(ts->NDW_root);
	}
}

static struct kideval_speakers *kideval_FindSpeaker(char *fname, char *sp, char *ID, char isSpeakerFound, struct database *db) {
	struct kideval_speakers *ts, *tsp;

	if (db == NULL)
		tsp = sp_head;
	else
		tsp = db->db_sp;
	uS.remblanks(sp);
	for (ts=tsp; ts != NULL; ts=ts->next_sp) {
		if (uS.mStricmp(ts->fname, fname) == 0) {
			if (uS.partcmp(ts->sp, sp, FALSE, FALSE)) {
				ts->isSpeakerFound = isSpeakerFound;
				return(ts);
			}
		}
	}
	if ((ts=NEW(struct kideval_speakers)) == NULL)
		kideval_error(db, TRUE);
	if ((ts->fname=(char *)malloc(strlen(fname)+1)) == NULL) {
		free(ts);
		kideval_error(db, TRUE);
	}	
	if (db == NULL) {
		if (sp_head == NULL) {
			sp_head = ts;
		} else {
			for (tsp=sp_head; tsp->next_sp != NULL; tsp=tsp->next_sp) ;
			tsp->next_sp = ts;
		}
		ts->next_sp = NULL;
	} else {
		ts->next_sp = db->db_sp;
		db->db_sp = ts;
	}
	strcpy(ts->fname, fname);
	if ((ts->sp=(char *)malloc(strlen(sp)+1)) == NULL) {
		kideval_error(db, TRUE);
	}
	strcpy(ts->sp, sp);
	if (ID == NULL || isCreateDB)
		ts->ID = NULL;
	else {
		changeAgeField(ID, templineC4);
		if ((ts->ID=(char *)malloc(strlen(templineC4)+1)) == NULL)
			kideval_error(db, TRUE);
		strcpy(ts->ID, templineC4);
	}
	ts->isSpeakerFound = isSpeakerFound;
	kideval_initTSVars(ts, TRUE);
	return(ts);
}

static void filterAndOuputID(char *line, char *IDText) {
	char *s, *sb;

	for (s=line; isSpace(*s); s++) ;
	sb = s;
	if ((s=strchr(sb, '|')) != NULL) {
	} else
		*sb = EOS;
#ifdef GEMS_COUNT
	fprintf(dbfpout, "+%s\n", line);
#else
	strcpy(IDText, line);
#endif
}
/*
static void dealWithDiscontinuousWord(char *word, int i) {
	if (word[strlen(word)-1] == '-') {
		while ((i=getword(utterance->speaker, uttline, templineC4, NULL, i))) {
			if (templineC4[0] == '-' && !uS.isToneUnitMarker(templineC4)) {
				strcat(word, templineC4+1);
				break;
			}
		}
	}
}
*/
static char isUttDel(char *line, int pos) {
	if (line[pos] == '?' && line[pos+1] == '|')
		;
	else if (uS.IsUtteranceDel(line, pos)) {
		if (!uS.atUFound(line, pos, &dFnt, MBF))
			return(TRUE);
	}
	return(FALSE);
}

static char isRightWord(char *word) {
	if (uS.mStricmp(word, "xxx") == 0       || uS.mStricmp(word, "yyy") == 0       || uS.mStrnicmp(word, "www", 3) == 0 ||
		uS.mStrnicmp(word, "xxx@s", 5) == 0 || uS.mStrnicmp(word, "yyy@s", 5) == 0 || uS.mStrnicmp(word, "www@s", 5) == 0 ||
		word[0] == '+' || word[0] == '-' || word[0] == '!' || word[0] == '?' || word[0] == '.')
		return(FALSE);
	return(TRUE);
}

static struct kideval_words *kideval_talloc(char *word, struct database *db, int wnum) {
	struct kideval_words *p;

	if ((p=NEW(struct kideval_words)) == NULL)
		kideval_error(db, TRUE);
	p->left = p->right = NULL;
	p->wnum = wnum;
	p->wused = TRUE;
	if ((p->word=(char *)malloc(strlen(word)+1)) != NULL)
		strcpy(p->word, word);
	else {
		free(p);
		kideval_error(db, TRUE);
	}
	return(p);
}

static struct kideval_words *kideval_FREQ_tree(struct kideval_words *p, char *w, struct kideval_speakers *ts, struct database *db) {
	int cond;
	struct kideval_words *t = p;

	if (p == NULL) {
		ts->wID++;
		ts->frTypes++;
		p = kideval_talloc(w, db, ts->wID);
	} else if ((cond=strcmp(w,p->word)) == 0) {
	} else if (cond < 0) {
		p->left = kideval_FREQ_tree(p->left, w, ts, db);
	} else {
		for (; (cond=strcmp(w,p->word)) > 0 && p->right != NULL; p=p->right) ;
		if (cond == 0) {
		} else if (cond < 0) {
			p->left = kideval_FREQ_tree(p->left, w, ts, db);
		} else {
			p->right = kideval_FREQ_tree(p->right, w, ts, db); /* if cond > 0 */
		}
		return(t);
	}
	return(p);
}

static struct kideval_words *kideval_NDW_tree(struct kideval_words *p, char *w, struct kideval_speakers *ts, struct database *db) {
	int cond;
	struct kideval_words *t = p;

	if (p == NULL) {
		ts->NDW++;
		p = kideval_talloc(w, db, ts->wID);
	} else if ((cond=strcmp(w,p->word)) == 0) {
	} else if (cond < 0) {
		p->left = kideval_NDW_tree(p->left, w, ts, db);
	} else {
		for (; (cond=strcmp(w,p->word)) > 0 && p->right != NULL; p=p->right) ;
		if (cond == 0) {
		} else if (cond < 0) {
			p->left = kideval_NDW_tree(p->left, w, ts, db);
		} else {
			p->right = kideval_NDW_tree(p->right, w, ts, db); /* if cond > 0 */
		}
		return(t);
	}
	return(p);
}

static int excludePlusAndUttDels(char *word) {
	if (word[0] == '+' || strcmp(word, "!") == 0 || strcmp(word, "?") == 0 || strcmp(word, ".") == 0)
		return(FALSE);
	return(TRUE);
}

static int countWord(char *line, int pos) {
	int  i, sqb, agb, num;
	char spf;

	for (i=pos-1; i >= 0 && (isSpace(line[i]) || line[i] == '\n'); i--) ;
	if (i < 0)
		return(0);
	sqb = 0;
	for (; i >= 0; i--) {
		if (line[i] == '[')
			sqb--;
		else if (line[i] == ']')
			sqb++;
		else if (sqb == 0 && (line[i] == '>' || !uS.isskip(line,i,&dFnt,MBF)))
			break;
	}
	if (i < 0)
		return(0);
	if (line[i] == '>') {
		num = 0;
		agb = 0;
		sqb = 0;
		spf = TRUE;
		for (; i >= 0; i--) {
			if (line[i] == '[') {
				sqb--;
				spf = TRUE;
			} else if (line[i] == ']') {
				sqb++;
				spf = TRUE;
			} else if (sqb == 0) {
				if (line[i] == '<') {
					agb--;
					spf = TRUE;
					if (agb <= 0)
						break;
				} else if (line[i] == '>') {
					agb++;
					spf = TRUE;
				} else if (!uS.isskip(line,i,&dFnt,MBF)) {
					if (spf)
						num++;
					spf = FALSE;
				} else
					spf = TRUE;
			}
		}
		if (i < 0)
			return(0);
		else
			return(num);
	} else {
		return(1);
	}
}

static float roundFloat(double num) {
	long t;

	t = (long)num;
	num = num - t;
	if (num > 0.5)
		t++;
	return(t);
}

static void kideval_process_tier(struct kideval_speakers *ts, struct database *db, char *rightIDFound, char *IDText) {
	int i, j, comps, num;
	char word[BUFSIZ+1], tword[1024], *w[NUMCOMPS], isMatchFound;
	char tmp, isWordsFound, isWordsFound100, sq, aq, isSkip;
	char isPSDFound, curPSDFound, isAuxFound, isAmbigFound;
	long stime, etime;
	double tNum;
	float mWords, morf, mUtt, mWords100, morf100, mUtt100;
	struct IDparts IDTier;
	COLS *col;
	MORFEATS word_feats, *clitic, *feat;

	if (utterance->speaker[0] == '*') {
		if (ts == NULL)
			return;
		strcpy(spareTier1, utterance->line);
		for (i=0; spareTier1[i] != EOS; i++) {
			if (spareTier1[i] == HIDEN_C && isdigit(spareTier1[i+1])) {
				if (getMediaTagInfo(spareTier1+i, &stime, &etime)) {
					tNum = etime;
					tNum = tNum - stime;
					tNum = tNum / 1000.0000;
					ts->tdur = ts->tdur + roundFloat(tNum);
				}
			}
		}
		ts->tdUtts = ts->tdUtts + 1;
		R6_override = TRUE;
		filterData("*", spareTier1);
		R6_override = FALSE;
		i = 0;
		while ((i=getword("*", spareTier1, word, NULL, i))) {
			if (word[0] == '-' && !uS.isToneUnitMarker(word) && !exclude(word))
				continue;
			if (exclude(word) && isRightWord(word)) {
				ts->tdWords = ts->tdWords + 1;
			}
		}			
		strcpy(spareTier1, utterance->line);
		for (i=0; utterance->line[i] != EOS; i++) {
			if ((i == 0 || uS.isskip(utterance->line,i-1,&dFnt,MBF)) && utterance->line[i] == '+' && 
				uS.isRightChar(utterance->line,i+1,',',&dFnt, MBF) && ts->isPSDFound) {
				if (ts->mUtt100 > 0.0)
					ts->mUtt100--;
				if (ts->mUtt > 0.0)
					ts->mUtt--;
				if (ts->tUtt > 0.0)
					ts->tUtt--;
				ts->isPSDFound = FALSE;
			}
		}
		i = 0;
		while ((i=getword(utterance->speaker, utterance->line, word, NULL, i))) {
			if (!strcmp(word, "[//]"))
				ts->retr++;
			else if (!strcmp(word, "[/]"))
				ts->repet++;
		}
	} else if (fDepTierName[0]!=EOS && uS.partcmp(utterance->speaker,fDepTierName,FALSE,FALSE)) {
		if (ts == NULL)
			return;
		filterwords("%", uttline, excludePlusAndUttDels);
		i = 0;
		while ((i=getword(utterance->speaker, uttline, templineC2, NULL, i))) {
			if (isWordFromMORTier(templineC2)) {
				if (ParseWordMorElems(templineC2, &word_feats) == FALSE)
					kideval_error(db, TRUE);
				for (clitic=&word_feats; clitic != NULL; clitic=clitic->clitc) {
					if (clitic->compd != NULL) {
						word[0] = EOS;
						for (feat=clitic; feat != NULL; feat=feat->compd) {
							if (feat->stem != NULL && feat->stem[0] != EOS) {
								strcat(word, feat->stem);
							}
						}
						if (word[0] != EOS) {
							if (ts->NDWTotal < 100) {
								ts->NDWTotal++;
								ts->NDW_root = kideval_NDW_tree(ts->NDW_root, word, ts, db);
							}
							if (exclude(word) && isRightWord(word)) {
								uS.remblanks(word);
								ts->frTokens++;
								ts->words_root = kideval_FREQ_tree(ts->words_root, word, ts, db);
							}
						}
					} else {
						if (clitic->stem != NULL && clitic->stem[0] != EOS) {
							if (ts->NDWTotal < 100) {
								ts->NDWTotal++;
								ts->NDW_root = kideval_NDW_tree(ts->NDW_root, clitic->stem, ts, db);
							}
							if (exclude(clitic->stem) && isRightWord(clitic->stem)) {
								uS.remblanks(clitic->stem);
								ts->frTokens++;
								ts->words_root = kideval_FREQ_tree(ts->words_root, clitic->stem, ts, db);
							}
						}
					}
				}
				freeUpFeats(&word_feats);
			}
		}
		i = 0;
		while ((i=getword(utterance->speaker, uttline, word, NULL, i))) {
			uS.remblanks(word);
			if (isWordFromMORTier(word)) {
				ts->morTotal++;
				strcpy(tword, word);
				for (j=0; tword[j] != EOS; j++) {
					if (tword[j] == '~' || tword[j] == '$')
						ts->morTotal++;
				}
				comps = 0;
				w[comps++] = tword;
				for (j=0; tword[j] != EOS; j++) {
					if (tword[j] == '~' || tword[j] == '$') {
						tword[j] = EOS;
						j++;
						if (comps >= NUMCOMPS) {
							fprintf(stderr, "    Intenal Error: # of pre/post clitics is > %d in word: \"%s\"\n", NUMCOMPS-1, word);
							sp_head = freespeakers(sp_head);
							DBKeyRoot = freeDBKeys(DBKeyRoot);
							labelsRoot = freeLabels(labelsRoot);
							colsRoot = freeColsP(colsRoot);
							db = freedb(db);
							dss_freeSpeakers();
							dss_cleanSearchMem();
							speakers = speaker_free_up_speakers(speakers, TRUE);
							if (VOCD_TYPE_D)
								denom_speakers = speaker_free_up_speakers(denom_speakers, TRUE);
							cutt_exit(0);
						}
						w[comps++] = tword+j;
					}
				}
				for (j=0; j < comps; j++) {
					for (col=colsRoot; col != NULL; col=col->next_col) {
						if (col->isClitic) {
							isMatchFound = isMorPatMatchedWord(col->pats, word);
							if (isMatchFound == 2)
								kideval_error(db, TRUE);
							if (j == 0 && isMatchFound) {
								if (col->labelP != NULL)
									ts->mor_count[col->labelP->num] = ts->mor_count[col->labelP->num] + 1.0;
							}
						} else {
							isMatchFound = isMorPatMatchedWord(col->pats, w[j]);
							if (isMatchFound == 2)
								kideval_error(db, TRUE);
							if (isMatchFound) {
								if (col->labelP != NULL)
									ts->mor_count[col->labelP->num] = ts->mor_count[col->labelP->num] + 1.0;
							}
						}
					}
				}
			}
		}
		isSkip = FALSE;
		if (isPostCodeOnUtt(spareTier1, "[+ mlue]")) {
			isSkip = TRUE;
		} else {
			i = 0;
			while ((i=getword(utterance->speaker, spareTier1, word, NULL, i))) {
				if (strcmp(word, "xxx") == 0 || strcmp(word, "yyy") == 0 || strcmp(word, "www") == 0) {
					isSkip = TRUE;
					break;
				}
			}
		}
		ts->isMORFound = TRUE;
		mWords100	= ts->mWords100;
		morf100		= ts->morf100;
		mUtt100		= ts->mUtt100;
		mWords		= ts->mWords;
		morf		= ts->morf;
		mUtt		= ts->mUtt;
		isPSDFound	= ts->isPSDFound;
		curPSDFound = FALSE;
		isWordsFound = FALSE;
		isWordsFound100 = FALSE;
		sq = FALSE;
		aq = FALSE;
		i = 0;
		do {
			if (uS.isRightChar(uttline, i, '[', &dFnt, MBF)) {
				sq = TRUE;
			} else if (uS.isRightChar(uttline, i, ']', &dFnt, MBF))
				sq = FALSE;
			else {
				if (uS.isRightChar(uttline, i, '<', &dFnt, MBF)) {
					int tl;
					for (tl=i+1; !uS.isRightChar(uttline, tl, '>', &dFnt, MBF) && uttline[tl]; tl++) {
						if (isdigit(uttline[tl])) ;
						else if (uttline[tl]== ' ' || uttline[tl]== '\t' || uttline[tl]== '\n') ;
						else if ((tl-1 == i+1 || !isalpha(uttline[tl-1])) && 
								 uS.isRightChar(uttline, tl, '-', &dFnt, MBF) && !isalpha(uttline[tl+1])) ;
						else if ((tl-1 == i+1 || !isalpha(uttline[tl-1])) && 
								 (uS.isRightChar(uttline,tl,'u', &dFnt, MBF) || uS.isRightChar(uttline,tl,'U', &dFnt, MBF)) && 
								 !isalpha(uttline[tl+1])) ;
						else if ((tl-1 == i+1 || !isalpha(uttline[tl-1])) && 
								 (uS.isRightChar(uttline,tl,'w', &dFnt, MBF) || uS.isRightChar(uttline,tl,'W', &dFnt, MBF)) && 
								 !isalpha(uttline[tl+1])) ;
						else if ((tl-1 == i+1 || !isalpha(uttline[tl-1])) && 
								 (uS.isRightChar(uttline,tl,'s', &dFnt, MBF) || uS.isRightChar(uttline,tl,'S', &dFnt, MBF)) && 
								 !isalpha(uttline[tl+1])) ;
						else
							break;
					}
					if (uS.isRightChar(uttline, tl, '>', &dFnt, MBF)) aq = TRUE;
				} else
					if (uS.isRightChar(uttline, i, '>', &dFnt, MBF)) aq = FALSE;
			}
			if (!uS.isskip(uttline,i,&dFnt,MBF) && !sq && !aq) {
				isAmbigFound = FALSE;
				isWordsFound = TRUE;
				isWordsFound100 = TRUE;
				tmp = TRUE;
				mWords = mWords + 1;
				mWords100 = mWords100 + 1;
				while (uttline[i]) {
					if (uttline[i] == '^')
						isAmbigFound = TRUE;
					if (uS.isskip(uttline,i,&dFnt,MBF)) {
						if (uS.IsUtteranceDel(utterance->line, i)) {
							if (!uS.atUFound(utterance->line, i, &dFnt, MBF))
								break;
						} else
							break;
					}
					if (!uS.ismorfchar(uttline, i, &dFnt, rootmorf, MBF) && !isAmbigFound) {
						if (tmp) {
							if (uttline[i] != EOS) {
								if (i >= 2 && uttline[i-1] == '+' && uttline[i-2] == '|')
									;
								else {
									morf = morf + 1;
									morf100 = morf100 + 1;
//									localmorf = localmorf + 1;
								}
							}
							tmp = FALSE;
						}
					} else
						tmp = TRUE;
					i++;
				}
			}
			if ((i == 0 || uS.isskip(utterance->line,i-1,&dFnt,MBF)) && utterance->line[i] == '+' && 
				uS.isRightChar(utterance->line,i+1,',',&dFnt, MBF) && isPSDFound) {
				if (mUtt100 > 0.0)
					mUtt100--;
				else if (mUtt100 == 0.0) 
					isWordsFound100 = FALSE;
				if (mUtt > 0.0)
					mUtt--;
				else if (mUtt == 0.0) 
					isWordsFound = FALSE;
				if (ts->tUtt > 0.0)
					ts->tUtt--;
				isPSDFound = FALSE;
			}
			if (isTierContSymbol(utterance->line, i, TRUE)) {
				if (mUtt100 > 0.0)
					mUtt100--;
				else if (mUtt100 == 0.0) 
					isWordsFound100 = FALSE;
				if (mUtt > 0.0)
					mUtt--;
				else if (mUtt == 0.0) 
					isWordsFound = FALSE;
				if (ts->tUtt > 0.0)
					ts->tUtt--;
			}
			if (isTierContSymbol(utterance->line, i, FALSE))
				curPSDFound = TRUE;
			if (isUttDel(utterance->line, i)) {
				if (uS.isRightChar(utterance->line, i, '[', &dFnt, MBF)) {
					for (; utterance->line[i] && !uS.isRightChar(utterance->line, i, ']', &dFnt, MBF); i++) ;
				}
				if (uS.isRightChar(utterance->line, i, ']', &dFnt, MBF))
					sq = FALSE;
				if (isWordsFound100) {
					mUtt100 = mUtt100 + (float)1.0;
					isWordsFound100 = FALSE;
				}
				if (isWordsFound) {
					mUtt = mUtt + (float)1.0;
					ts->tUtt = ts->tUtt + (float)1.0;
					isWordsFound = FALSE;
				}
			}
			if (uttline[i])
				i++;
			else
				break;
		} while (uttline[i]) ;
		if (!isSkip) {
			if (mUtt100 <= 100) {
				ts->mWords100	= mWords100;
				ts->morf100		= morf100;
				ts->mUtt100		= mUtt100;
			}
			ts->mWords	   = mWords;
			ts->morf	   = morf;
			ts->mUtt	   = mUtt;
			ts->isPSDFound = curPSDFound;
		}
		for (i=0; spareTier1[i] != EOS; i++) {
			if (spareTier1[i] == '[' && spareTier1[i+1] == '*' && (spareTier1[i+2] == ']' || spareTier1[i+2] == ' ')) {
				num = countWord(spareTier1, i);
				if (num > 0)
					ts->werr += num;
				else
					ts->werr++;
			}
		}
		i = 0;
		while ((i=getword("*", spareTier1, word, NULL, i))) {
			if (uS.isSqCodes(word, tword, &dFnt, FALSE)) {
				if (!strcmp(tword, "[+ *]")   || !strcmp(tword, "[+ jar]") || !strcmp(tword, "[+ es]") ||
					!strcmp(tword, "[+ cir]") || !strcmp(tword, "[+ per]") || !strcmp(tword, "[+ gram]")) {
					ts->uerr++;
				}
			}
		}
		isAuxFound = FALSE;
		i = 0;
		while ((i=getword(utterance->speaker, uttline, templineC2, NULL, i))) {
			if (isWordFromMORTier(templineC2)) {
				if (ParseWordMorElems(templineC2, &word_feats) == FALSE)
					kideval_error(db, TRUE);
				for (clitic=&word_feats; clitic != NULL; clitic=clitic->clitc) {
					for (feat=clitic; feat != NULL; feat=feat->compd) {
						if (feat->type != NULL && feat->typeID == '+')
							;
						else if (isAllv(feat) || isnEqual("cop", feat->pos, 3) || isnEqual("mod", feat->pos, 3)) {
							if (isEqual("part", feat->pos)) {
								if (isAuxFound)
									ts->CUR++;
							} else if (isEqual("v", feat->pos) || isnEqual("cop", feat->pos, 3))
								ts->CUR++;
						} else if (isEqual("aux", feat->pos)) {
							isAuxFound = TRUE;
						} else if (isEqual("part", feat->pos)) {
							if (isAuxFound)
								ts->CUR++;
						}
					}
				}
				freeUpFeats(&word_feats);
			}
		}
	} else if (uS.partcmp(utterance->speaker,"@ID:",FALSE,FALSE)) {
		if (db == NULL) {
			if (isIDSpeakerSpecified(utterance->line, templineC, TRUE)) {
				if (isCreateDB) {
					uS.remblanks(utterance->line);
					filterAndOuputID(utterance->line, IDText);
				} else {
					uS.remblanks(utterance->line);
					kideval_FindSpeaker(oldfname, templineC, utterance->line, FALSE, db);
				}
			}
		} else if (rightIDFound != NULL) {
			uS.remblanks(utterance->line);
			breakIDsIntoFields(&IDTier, utterance->line);
			if (isKeyMatch(&IDTier))  {
				*rightIDFound = TRUE;
				maketierchoice(IDTier.code, '+', '\002');
			}
		}
	}
}

static char kideval_process_dss_tier(struct kideval_speakers *ts, struct database *db, char lRightspeaker, DSSSP **lastdsp) {
	int k;
	DSSSP *dss_sp;

	if (utterance->speaker[0] == '*') {
		if (ts == NULL)
			return(lRightspeaker);

		if (*lastdsp != NULL) {
			freeLastDSSUtt(*lastdsp);
			*lastdsp = NULL;
		}
		dss_sp = dss_FindSpeaker(ts->sp, NULL);
		if (dss_sp == NULL)
			kideval_error(db, TRUE);
		if (isUttsLimit(dss_sp)) {
			spareTier1[0] = EOS;
			return(FALSE);
		}
		cleanUttline(uttline);
		if (strcmp(uttline, spareTier1) == 0) {
			spareTier1[0] = EOS;
			return(FALSE);
		}
		if (!isDSSRepeatUtt(dss_sp, uttline)) {
			if (!addToDSSUtts(dss_sp, uttline)) {
				kideval_error(db, TRUE);
			}
			strcpy(spareTier1, uttline);
			strcpy(spareTier2, utterance->line);
			*lastdsp = dss_sp;
		} else {
			spareTier1[0] = EOS;
			lRightspeaker = FALSE;
		}
	} else if (fDepTierName[0]!=EOS && uS.partcmp(utterance->speaker,fDepTierName,FALSE,FALSE)) {
		if (ts == NULL)
			return(lRightspeaker);
		dss_sp = dss_FindSpeaker(ts->sp, NULL);
		if (dss_sp == NULL)
			kideval_error(db, TRUE);
		ts->dss_sp = dss_sp;
		for (k=0; uttline[k] != EOS; k++) {
			if (uttline[k] == '~' || uttline[k] == '$')
				uttline[k] = ' ';
		}
		if (!make_DSS_tier(uttline))
			kideval_error(db, FALSE);
		*lastdsp = NULL;
		if (PassedDSSMorTests(spareTier2,uttline,templineC3)) {
			dss_sp->TotalNum = dss_sp->TotalNum + 1;
			for (k=0; spareTier2[k] != EOS; k++) {
				if (spareTier2[k]== '\n' || spareTier2[k]== '\t')
					spareTier2[k] = ' ';
			}
			for (k--; spareTier2[k] == ' '; k--) ;
			spareTier2[++k] = EOS;
			dss_sp->GrandTotal = dss_sp->GrandTotal + PrintOutDSSTable(spareTier2, FALSE);
		} else {
			dss_sp = dss_FindSpeaker(ts->sp, NULL);
			if (dss_sp == NULL)
				kideval_error(db, TRUE);
			freeLastDSSUtt(dss_sp);
		}
		spareTier1[0] = EOS;
	}
	return(lRightspeaker);
}

static void kideval_process_vocd_tier(struct kideval_speakers *ts, struct database *db) {
	int  i;
	char word[BUFSIZ+1];
	MORFEATS word_feats, *clitic, *feat;
	VOCDSP *vocd_sp;

	if (utterance->speaker[0] == '*') {
/*
		if (ts == NULL)
			return;
		isDoVocdAnalises = TRUE;
		if ((vocd_sp=speaker_add_speaker(&speakers, ts->sp, NULL, NULL, TRUE, TRUE)) == NULL) {
			fprintf(stderr,"cannot add speaker to list\n");
			kideval_error(db, FALSE);
		}
		ts->vocd_sp = vocd_sp;
		if (!nomain && vocd_sp != NULL) {
			uS.remblanks(uttline);
			for (i=0; uttline[i] != EOS; i++) {
				if (uS.isskip(uttline,i,&dFnt,MBF))
					uttline[i] = ' ';
			}
			if (!speaker_add_line(vocd_sp, uttline, lineno)) {
				fprintf(stderr,"cannot add line to current speaker\n");
				kideval_error(db, FALSE);
			}
		}
*/
	} else if (fDepTierName[0]!=EOS && uS.partcmp(utterance->speaker,fDepTierName,FALSE,FALSE)) {
		if (ts == NULL)
			return;
		isDoVocdAnalises = TRUE;
		if ((vocd_sp=speaker_add_speaker(&speakers, ts->sp, NULL, NULL, TRUE, TRUE)) == NULL) {
			fprintf(stderr,"cannot add speaker to list\n");
			kideval_error(db, FALSE);
		}
		ts->vocd_sp = vocd_sp;
		if (vocd_sp != NULL) {
			uS.remblanks(uttline);
			filterwords("%", uttline, excludePlusAndUttDels);
			templineC1[0] = EOS;
			i = 0;
			while ((i=getword(utterance->speaker, uttline, templineC2, NULL, i))) {
				if (isWordFromMORTier(templineC2)) {
					if (ParseWordMorElems(templineC2, &word_feats) == FALSE)
						kideval_error(db, TRUE);
					for (clitic=&word_feats; clitic != NULL; clitic=clitic->clitc) {
						if (clitic->compd != NULL) {
							word[0] = EOS;
							for (feat=clitic; feat != NULL; feat=feat->compd) {
								if (feat->stem != NULL && feat->stem[0] != EOS) {
									if (word[0] != EOS)
										strcat(word, "+");
									strcat(word, feat->stem);
								}
							}
							if (word[0] != EOS) {
								if (exclude(word) && isRightWord(word)) {
									uS.remblanks(word);
									strcat(templineC1, word);
									strcat(templineC1, " ");
								}
							}
						} else {
							if (clitic->stem != NULL && clitic->stem[0] != EOS) {
								if (exclude(clitic->stem) && isRightWord(clitic->stem)) {
									uS.remblanks(clitic->stem);
									strcat(templineC1, clitic->stem);
									strcat(templineC1, " ");
								}
							}
						}
					}
					freeUpFeats(&word_feats);
				}
			}
			strcpy(uttline, templineC1);
			for (i=0; uttline[i] != EOS; i++) {
				if (uS.isskip(uttline,i,&dFnt,MBF))
					uttline[i] = ' ';
			}
			if (!speaker_add_line(vocd_sp, uttline, lineno)) {
				fprintf(stderr,"cannot add line to current speaker\n");
				kideval_error(db, FALSE);
			}
		}
	}
}

static char kideval_process_ipsyn_tier(struct kideval_speakers *ts, struct database *db, char lRightspeaker, int *misalignmentError, IPSYN_SP **lastipsp) {
	char isFC;
	IPSYN_SP *ipsyn_sp;

	if (utterance->speaker[0] == '*') {
		if (ts == NULL)
			return(lRightspeaker);
		if (*lastipsp != NULL) {
			removeLastUtt(*lastipsp);
			*lastipsp = NULL;
		}
		ipsyn_sp = ipsyn_FindSpeaker(ts->sp, NULL, TRUE);
		if (ipsyn_sp == NULL)
			kideval_error(db, FALSE);
		if (ipsyn_sp->UttNum >= IPS_UTTLIM) {
			lRightspeaker = FALSE;
		} else {
			cleanUttline(uttline);
			isFC = forceInclude(utterance->line);
			if (!isFC && (strcmp(uttline, spareTier1) == 0 || isExcludeIpsynTier(utterance->line, uttline))) {
				spareTier1[0] = EOS;
				lRightspeaker = FALSE;
			} else {
				if (!isIPRepeatUtt(ipsyn_sp, uttline) || isFC) {
					ipsyn_sp->UttNum++;
					add2Utts(ipsyn_sp, uttline);
					strcpy(spareTier1, uttline);
					*lastipsp = ipsyn_sp;
				} else {
					spareTier1[0] = EOS;
					lRightspeaker = FALSE;
				}
			}
		}
	} else if (fDepTierName[0] != EOS && uS.partcmp(utterance->speaker,fDepTierName,FALSE,FALSE)) {
		if (ts == NULL)
			return(lRightspeaker);
		*lastipsp = NULL;
		ipsyn_sp = ipsyn_FindSpeaker(ts->sp, NULL, TRUE);
		if (ipsyn_sp == NULL)
			kideval_error(db, FALSE);
		ts->ipsyn_sp = ipsyn_sp;
		strcpy(utterance->tuttline,utterance->line);
		createMorUttline(utterance->line, org_spTier, "%mor:", utterance->tuttline, TRUE, TRUE);
		strcpy(utterance->tuttline, utterance->line);
		if (strchr(uttline, dMarkChr) == NULL) {
			if (isCreateDB) {
				fprintf(stderr,"\n*** File \"%s\": line %ld.\n", oldfname, lineno);
				fprintf(stderr,"WARNING: %s tier does not have one-to-one correspondence to its speaker tier.\n", fDepTierName);
				fprintf(stderr,"%s%s%%mor tier: %s\n", org_spName, org_spTier, utterance->line);
				kideval_error(db, FALSE);
			}
			*misalignmentError = *misalignmentError + 1;
			return(lRightspeaker);
		}
		if (ipsyn_sp->cUtt == NULL) {
			fprintf(stderr,"\n*** File \"%s\": line %ld.\n", oldfname, lineno);
			fprintf(stderr,"INTERNAL ERROR utt == null");
			kideval_error(db, FALSE);
		}
		ipsyn_sp->cUtt->morLine = (char *)malloc(strlen(utterance->tuttline)+1);
		if (ipsyn_sp->cUtt->morLine == NULL) {
			fprintf(stderr,"Error: out of memory\n");
			kideval_error(db, FALSE);
		}
		strcpy(ipsyn_sp->cUtt->morLine, utterance->tuttline);
		spareTier1[0] = EOS;
	} else if (uS.partcmp(utterance->speaker,"%gra:",FALSE,FALSE)) {
		if (ts == NULL)
			return(lRightspeaker);
		ipsyn_sp = ipsyn_FindSpeaker(ts->sp, NULL, TRUE);
		if (ipsyn_sp == NULL)
			kideval_error(db, FALSE);
		ts->ipsyn_sp = ipsyn_sp;
		if (ipsyn_sp->cUtt == NULL) {
			fprintf(stderr,"\n*** File \"%s\": line %ld.\n", oldfname, lineno);
			fprintf(stderr,"INTERNAL ERROR utt == null");
			kideval_error(db, FALSE);
		}
		ipsyn_sp->cUtt->graLine = (char *)malloc(strlen(utterance->tuttline)+1);
		if (ipsyn_sp->cUtt->graLine == NULL) {
			fprintf(stderr,"Error: out of memory\n");
			kideval_error(db, FALSE);
		}
		strcpy(ipsyn_sp->cUtt->graLine, utterance->tuttline);
	}
	return(lRightspeaker);
}

#ifdef GEMS_COUNT
static void kideval_outputtree(struct kideval_words *p) {
	if (p != NULL) {
		kideval_outputtree(p->left);
		if (p->wused)
			fprintf(dbfpout, "%d ", p->wnum);
//			fprintf(dbfpout, "%s ", p->word);
		kideval_outputtree(p->right);
	}
}
#endif

static char *extractFloatFromLine(char *line, float *val, struct database *db) {
	float num;

	if (*line == EOS) {
		fprintf(stderr, "\nERROR: Database is corrupt\n");
		kideval_error(db, FALSE);
	}
	num = atof(line);
	*val = *val + num;
	while (isdigit(*line) || *line == '.')
		line++;
	while (isSpace(*line))
		line++;
	return(line);
}

static void retrieveTS(struct kideval_speakers *ts, char *line, struct database *db) {
	float tf;
	struct label_cols *col;

	while (isSpace(*line))
		line++;

	line = extractFloatFromLine(line, &ts->mWords, db);
	line = extractFloatFromLine(line, &ts->morf, db);
	line = extractFloatFromLine(line, &ts->tUtt, db);
	line = extractFloatFromLine(line, &ts->mUtt, db);

	line = extractFloatFromLine(line, &ts->mWords100, db);
	line = extractFloatFromLine(line, &ts->morf100, db);
	line = extractFloatFromLine(line, &ts->mUtt100, db);

	line = extractFloatFromLine(line, &ts->frTokens, db);
#ifndef GEMS_COUNT
	line = extractFloatFromLine(line, &ts->frTypes, db);
	line = extractFloatFromLine(line, &ts->NDW, db);
	line = extractFloatFromLine(line, &ts->NDWTotal, db);
#endif
	line = extractFloatFromLine(line, &ts->vocdOptimum, db);
	line = extractFloatFromLine(line, &ts->CUR, db);


	line = extractFloatFromLine(line, &ts->tdWords, db);
	line = extractFloatFromLine(line, &ts->tdUtts, db);
	line = extractFloatFromLine(line, &ts->tdur, db);

	line = extractFloatFromLine(line, &ts->werr, db);
	line = extractFloatFromLine(line, &ts->uerr, db);

	line = extractFloatFromLine(line, &ts->retr, db);
	line = extractFloatFromLine(line, &ts->repet, db);

	line = extractFloatFromLine(line, &ts->dssUttCnt, db);
	line = extractFloatFromLine(line, &ts->dssGTotal, db);

	line = extractFloatFromLine(line, &ts->ipsynUttCnt, db);
	line = extractFloatFromLine(line, &ts->ipsynScore, db);

	line = extractFloatFromLine(line, &ts->morTotal, db);

	for (col=labelsRoot; col != NULL; col=col->next_label) {
		if (ts->mor_count == NULL) {
			line = extractFloatFromLine(line, &tf, db);
		} else {
			tf = ts->mor_count[col->num];
			line = extractFloatFromLine(line, &tf, db);
			ts->mor_count[col->num] = tf;
		}
	}

#ifdef GEMS_COUNT
	while (*line != EOS) {
		e = strchr(line, ' ');
		if (e != NULL)
			*e = EOS;
		if (!strcmp(line, ".NDW.")) {
			if (e == NULL) {
				for (; *line != ' ' && *line != EOS; line++) ;
			} else {
				line = e + 1;
			}
			while (isSpace(*line))
				line++;
			break;
		}
		ts->words_root = kideval_FREQ_tree(ts->words_root, line, ts, db);
		if (e == NULL)
			break;
		line = e + 1;
		while (isSpace(*line))
			line++;
	}
	while (*line != EOS) {
		e = strchr(line, ' ');
		if (e != NULL)
			*e = EOS;
		ts->NDW_root = kideval_NDW_tree(ts->NDW_root, line, ts, db);
		if (e == NULL)
			break;
		line = e + 1;
		while (isSpace(*line))
			line++;
	}
#endif
}

#ifdef DEBUGEVALRESULTS
static void prDebug(struct kideval_speakers *ts, char *IDText, char *fileName) {
	struct label_cols *col;

	if (ts == NULL)
		return;
	if (dbResults == NULL) {
		dbResults = fopen(DATABASEFILESDEBUG, "wb");
		if (dbResults == NULL) {
			fprintf(stderr,"Can't create file \"%s\", perhaps path doesn't exist or it is opened by another application\n", "DB_results.txt");
			kideval_error(NULL, FALSE);
		}
	}
	fprintf(dbResults, "*** File=%s\n", fileName);
	fprintf(dbResults, "@ID=%s\n", IDText);

	fprintf(dbResults, "mWords=%.0f\n", ts->mWords);
	fprintf(dbResults, "morf=%.0f\n", ts->morf);
	fprintf(dbResults, "tUtt=%.0f\n", ts->tUtt);
	fprintf(dbResults, "mUtt=%.0f\n", ts->mUtt);

	fprintf(dbResults, "mWords100=%.0f\n", ts->mWords100);
	fprintf(dbResults, "morf100=%.0f\n", ts->morf100);
	fprintf(dbResults, "mUtt100=%.0f\n", ts->mUtt100);

	fprintf(dbResults, "frTokens=%.0f\n", ts->frTokens);
#ifndef GEMS_COUNT
	fprintf(dbResults, "frTypes=%.0f\n", ts->frTypes);
	fprintf(dbResults, "NDW=%.0f\n", ts->NDW);
	fprintf(dbResults, "NDW=%.0f\n", ts->NDWTotal);
#else
	fprintf(dbResults, "    frTypes=%.0f\n", ts->frTypes);
	fprintf(dbResults, "    NDW=%.0f\n", ts->NDW);
	fprintf(dbResults, "    NDW=%.0f\n", ts->NDWTotal);
#endif
	fprintf(dbResults, "vocdOptimum=%f\n", ts->vocdOptimum);
	fprintf(dbResults, "CUR=%.0f\n", ts->CUR);

	fprintf(dbResults, "tdWords=%.0f\n", ts->tdWords);
	fprintf(dbResults, "tdUtts=%.0f\n", ts->tdUtts);
	fprintf(dbResults, "tdur=%.0f\n", ts->tdur);

	fprintf(dbResults, "werr=%.0f\n", ts->werr);
	fprintf(dbResults, "uerr=%.0f\n", ts->uerr);

	fprintf(dbResults, "retr=%.0f\n", ts->retr);
	fprintf(dbResults, "repet=%.0f\n", ts->repet);

	fprintf(dbResults, "dssUttCnt=%.0f\n", ts->dssUttCnt);
	fprintf(dbResults, "dssGTotal=%.0f\n", ts->dssGTotal);

	fprintf(dbResults, "ipsynUttCnt=%.0f\n", ts->ipsynUttCnt);
	fprintf(dbResults, "ipsynScore=%.0f\n", ts->ipsynScore);

	fprintf(dbResults, "morTotal=%.0f\n", ts->morTotal);

	for (col=labelsRoot; col != NULL; col=col->next_label) {
		if (ts->mor_count == NULL) {
			fprintf(dbResults, "mor_count[%d]=NULL\n", col->num);
		} else {
			fprintf(dbResults, "mor_count[%d]=%.0f\n", col->num, ts->mor_count[col->num]);
		}
	}
}
#endif

static void prTSResults(struct kideval_speakers *ts, char *IDText) {
	const char *format, *fn;
	struct label_cols *col;

	format = "%.0f ";
//	format = "%f ";

#ifndef GEMS_COUNT
	fn = strchr(oldfname+wdOffset, PATHDELIMCHR);
	if (fn != NULL)
		fn++;
	else
		fn = oldfname+wdOffset;
	fprintf(dbfpout, "=%s\n", fn);
	fprintf(dbfpout, "+%s\n", IDText);
#endif
	fprintf(dbfpout, format, ts->mWords);
	fprintf(dbfpout, format, ts->morf);
	fprintf(dbfpout, format, ts->tUtt);
	fprintf(dbfpout, format, ts->mUtt);

	fprintf(dbfpout, format, ts->mWords100);
	fprintf(dbfpout, format, ts->morf100);
	fprintf(dbfpout, format, ts->mUtt100);

	fprintf(dbfpout, format, ts->frTokens);
#ifndef GEMS_COUNT
	fprintf(dbfpout, format, ts->frTypes);
	fprintf(dbfpout, format, ts->NDW);
	fprintf(dbfpout, format, ts->NDWTotal);
#endif
	fprintf(dbfpout, "%f ", ts->vocdOptimum);
	fprintf(dbfpout, format, ts->CUR);

	fprintf(dbfpout, format, ts->tdWords);
	fprintf(dbfpout, format, ts->tdUtts);
	fprintf(dbfpout, format, ts->tdur);

	fprintf(dbfpout, format, ts->werr);
	fprintf(dbfpout, format, ts->uerr);

	fprintf(dbfpout, format, ts->retr);
	fprintf(dbfpout, format, ts->repet);

	fprintf(dbfpout, format, ts->dssUttCnt);
	fprintf(dbfpout, format, ts->dssGTotal);

	fprintf(dbfpout, format, ts->ipsynUttCnt);
	fprintf(dbfpout, format, ts->ipsynScore);

	fprintf(dbfpout, format, ts->morTotal);

	for (col=labelsRoot; col != NULL; col=col->next_label) {
		if (ts->mor_count == NULL) {
			fprintf(dbfpout, format, 0.0);
		} else {
			fprintf(dbfpout, format, ts->mor_count[col->num]);
		}
	}

#ifdef GEMS_COUNT
	kideval_outputtree(ts->words_root);
	fprintf(dbfpout, ".NDW. ");
	kideval_outputtree(ts->NDW_root);
#endif
	putc('\n', dbfpout);
}

static int isRightText(char *gem_word) {
	int i = 0;
	int found = 0;

	if (GemMode == '\0')
		return(TRUE);
	filterwords("@", uttline, excludeGemKeywords);
	while ((i=getword(utterance->speaker, uttline, gem_word, NULL, i)))
		found++;
	if (GemMode == 'i') 
		return((kideval_group == FALSE && found) || (kideval_SpecWords == found));
	else 
		return((kideval_group == TRUE && kideval_SpecWords > found) || (found == 0));
}

static void OpenDBFile(FILE *fp, struct database *db, char *isDbSpFound) {
	int  found, t;
	char isOutputGem, rightIDFound;
	char word[BUFSIZ+1], DBFname[BUFSIZ+1], IDText[BUFSIZ+1];
	float tt;
	FILE *tfpin;
	struct kideval_speakers *ts;
	struct IDparts IDTier;
	struct label_cols *col;

	tfpin = fpin;
	fpin = fp;
	ts = NULL;
	if (isNOptionSet == FALSE) {
		strcpy(kideval_BBS, "@*&#");
		strcpy(kideval_CBS, "@*&#");
	}
	if (kideval_SpecWords) {
		isOutputGem = FALSE;
	} else {
		isOutputGem = TRUE;
	}
	if (!fgets_cr(templineC, UTTLINELEN, fpin))
		return;
	if (uS.isUTF8(templineC)) {
		if (!fgets_cr(templineC, UTTLINELEN, fpin))
			return;
	}
	if (templineC[0] != 'V' || !isdigit(templineC[1])) {
		fprintf(stderr, "\nERROR: Selected database is corrupted.\n");
		fprintf(stderr, "Please download new database with \"File->Get KIDEVAL Database\" menu\n");
		kideval_error(db, FALSE);
	}
	t = atoi(templineC+1);
	if (t != KIDEVAL_DB_VERSION) {
		fprintf(stderr, "\nERROR: Selected database is incompatible with this version of CLAN.\n");
		if (t > KIDEVAL_DB_VERSION) {
			fprintf(stderr, "Please download a new CLAN from talkbank.org/clan server and re-install it.\n\n");
		} else {
			fprintf(stderr, "Please download new database with \"File->Get KIDEVAL Database\" menu\n");
		}
		kideval_error(db, FALSE);
	}
	for (t=1; isdigit(templineC[t]); t++) ;
	for (; isSpace(templineC[t]); t++) ;
	strncpy(DB_version, templineC+t, 64);
	DB_version[64] = EOS;
	uS.remblanks(DB_version);
	spareTier1[0] = EOS;
	rightIDFound = FALSE;
	found = 0;
	IDText[0] = EOS;
	word[0] = EOS;
	DBFname[0] = EOS;
	while (fgets_cr(templineC, UTTLINELEN, fpin)) {
		if (templineC[0] == '=') {
			uS.remblanks(templineC+1);
			if (templineC[1] == '/')
				strncpy(DBFname, templineC+24, BUFSIZ);
			else
				strncpy(DBFname, templineC+1, BUFSIZ);
			DBFname[BUFSIZ] = EOS;
		} else if (templineC[0] == '+') {
			uS.remblanks(templineC+1);
			strncpy(IDText, templineC+1, BUFSIZ);
			IDText[BUFSIZ] = EOS;
			breakIDsIntoFields(&IDTier, templineC+1);
			if (!isKeyMatch(&IDTier))  {
				while (fgets_cr(templineC, UTTLINELEN, fpin)) {
					if (templineC[0] == '-') {
						break;
					}
				}
				rightIDFound = FALSE;
			} else
				rightIDFound = TRUE;
		} else if (templineC[0] == '-') {
			ts = db->db_sp;
			if (ts != NULL) {
				if (!ts->isSpeakerFound) {
					if (kideval_SpecWords)
						fprintf(stderr,"\nERROR: No specified Gems found in database \"%s\"\n\n", FileName1);
					else
						fprintf(stderr,"\nERROR: No speaker matching +d option found in database \"%s\"\n\n", FileName1);
					kideval_error(db, FALSE);
				}
				*isDbSpFound = TRUE;
#ifdef DEBUGEVALRESULTS
				prDebug(ts, IDText, DBFname);
#endif
				if (ts->tUtt >= 50.0) {
					if (DBFilesListFP != NULL) {
						fprintf(DBFilesListFP, "-----\t%s\n", DBFname);
						fprintf(DBFilesListFP, "@ID:\t%s\n", IDText);
					}

					db->tUtt_num++;
					db->tUtt_sqr = db->tUtt_sqr + (ts->tUtt * ts->tUtt);
					db->tUtt = db->tUtt + ts->tUtt;
					if (db->mn_tUtt == (float)0.0 || db->mn_tUtt > ts->tUtt)
						db->mn_tUtt = ts->tUtt;
					if (db->mx_tUtt < ts->tUtt)
						db->mx_tUtt = ts->tUtt;
					db->frTokens_num++;
					db->frTokens_sqr = db->frTokens_sqr + (ts->frTokens * ts->frTokens);
					db->frTokens = db->frTokens + ts->frTokens;
					if (db->mn_frTokens == (float)0.0 || db->mn_frTokens > ts->frTokens)
						db->mn_frTokens = ts->frTokens;
					if (db->mx_frTokens < ts->frTokens)
						db->mx_frTokens = ts->frTokens;

					db->frTypes_num++;
					db->frTypes_sqr = db->frTypes_sqr + (ts->frTypes * ts->frTypes);
					db->frTypes = db->frTypes + ts->frTypes;
					if (db->mn_frTypes == (float)0.0 || db->mn_frTypes > ts->frTypes)
						db->mn_frTypes = ts->frTypes;
					if (db->mx_frTypes < ts->frTypes)
						db->mx_frTypes = ts->frTypes;
					if (ts->NDWTotal >= 100.0) {
						db->NDW_num++;
						db->NDW_sqr = db->NDW_sqr + (ts->NDW * ts->NDW);
						db->NDW = db->NDW + ts->NDW;
						if (db->mn_NDW == (float)0.0 || db->mn_NDW > ts->NDW)
							db->mn_NDW = ts->NDW;
						if (db->mx_NDW < ts->NDW)
							db->mx_NDW = ts->NDW;
					}
					if (ts->frTokens == 0.0)
						tt = 0.0;
					else
						tt = ts->frTypes / ts->frTokens;
					db->TTR_num++;
					db->TTR_sqr = db->TTR_sqr + (tt * tt);
					db->TTR = db->TTR + tt;
					if (db->mn_TTR == (float)0.0 || db->mn_TTR > tt)
						db->mn_TTR = tt;
					if (db->mx_TTR < tt)
						db->mx_TTR = tt;
					if (ts->tUtt == 0.0)
						tt = 0.0;
					else
						tt = ts->CUR / ts->tUtt;
					db->CUR_num++;
					db->CUR_sqr = db->CUR_sqr + (tt * tt);
					db->CUR = db->CUR + tt;
					if (db->mn_CUR == (float)0.0 || db->mn_CUR > tt)
						db->mn_CUR = tt;
					if (db->mx_CUR < tt)
						db->mx_CUR = tt;
					if (ts->mUtt > 0) {
						db->mUtt_num++;
						db->mUtt_sqr = db->mUtt_sqr + (ts->mUtt * ts->mUtt);
						db->mUtt = db->mUtt + ts->mUtt;
						if (db->mn_mUtt == (float)0.0 || db->mn_mUtt > ts->mUtt)
							db->mn_mUtt = ts->mUtt;
						if (db->mx_mUtt < ts->mUtt)
							db->mx_mUtt = ts->mUtt;

						ts->mWords = ts->mWords / ts->mUtt;
						db->mWords_num++;
						db->mWords_sqr = db->mWords_sqr + (ts->mWords * ts->mWords);
						db->mWords = db->mWords + ts->mWords;
						if (db->mn_mWords == (float)0.0 || db->mn_mWords > ts->mWords)
							db->mn_mWords = ts->mWords;
						if (db->mx_mWords < ts->mWords)
							db->mx_mWords = ts->mWords;

						ts->morf = ts->morf / ts->mUtt;
						db->morf_num++;
						db->morf_sqr = db->morf_sqr + (ts->morf * ts->morf);
						db->morf = db->morf + ts->morf;
						if (db->mn_morf == (float)0.0 || db->mn_morf > ts->morf)
							db->mn_morf = ts->morf;
						if (db->mx_morf < ts->morf)
							db->mx_morf = ts->morf;
					}
					if (ts->mUtt100 >= 100.0) {
						db->mUtt100_num++;
						db->mUtt100_sqr = db->mUtt100_sqr + (ts->mUtt100 * ts->mUtt100);
						db->mUtt100 = db->mUtt100 + ts->mUtt100;
						if (db->mn_mUtt100 == (float)0.0 || db->mn_mUtt100 > ts->mUtt100)
							db->mn_mUtt100 = ts->mUtt100;
						if (db->mx_mUtt100 < ts->mUtt100)
							db->mx_mUtt100 = ts->mUtt100;

						ts->mWords100 = ts->mWords100 / ts->mUtt100;
						db->mWords100_num++;
						db->mWords100_sqr = db->mWords100_sqr + (ts->mWords100 * ts->mWords100);
						db->mWords100 = db->mWords100 + ts->mWords100;
						if (db->mn_mWords100 == (float)0.0 || db->mn_mWords100 > ts->mWords100)
							db->mn_mWords100 = ts->mWords100;
						if (db->mx_mWords100 < ts->mWords100)
							db->mx_mWords100 = ts->mWords100;

						ts->morf100 = ts->morf100 / ts->mUtt100;
						db->morf100_num++;
						db->morf100_sqr = db->morf100_sqr + (ts->morf100 * ts->morf100);
						db->morf100 = db->morf100 + ts->morf100;
						if (db->mn_morf100 == (float)0.0 || db->mn_morf100 > ts->morf100)
							db->mn_morf100 = ts->morf100;
						if (db->mx_morf100 < ts->morf100)
							db->mx_morf100 = ts->morf100;
					}
					db->tdWords_num++;
					db->tdWords_sqr = db->tdWords_sqr + (ts->tdWords * ts->tdWords);
					db->tdWords = db->tdWords + ts->tdWords;
					if (db->mn_tdWords == (float)0.0 || db->mn_tdWords > ts->tdWords)
						db->mn_tdWords = ts->tdWords;
					if (db->mx_tdWords < ts->tdWords)
						db->mx_tdWords = ts->tdWords;

					db->tdUtts_num++;
					db->tdUtts_sqr = db->tdUtts_sqr + (ts->tdUtts * ts->tdUtts);
					db->tdUtts = db->tdUtts + ts->tdUtts;
					if (db->mn_tdUtts == (float)0.0 || db->mn_tdUtts > ts->tdUtts)
						db->mn_tdUtts = ts->tdUtts;
					if (db->mx_tdUtts < ts->tdUtts)
						db->mx_tdUtts = ts->tdUtts;

					db->tdur_num++;
					db->tdur_sqr = db->tdur_sqr + (ts->tdur * ts->tdur);
					db->tdur = db->tdur + ts->tdur;
					if (db->mn_tdur == (float)0.0 || db->mn_tdur > ts->tdur)
						db->mn_tdur = ts->tdur;
					if (db->mx_tdur < ts->tdur)
						db->mx_tdur = ts->tdur;

					if (ts->tdur == 0.0)
						tt = 0.0;
					else
						tt = ts->tdWords / ts->tdur;
					db->tdWT_num++;
					db->tdWT_sqr = db->tdWT_sqr + (tt * tt);
					db->tdWT = db->tdWT + tt;
					if (db->mn_tdWT == (float)0.0 || db->mn_tdWT > tt)
						db->mn_tdWT = tt;
					if (db->mx_tdWT < tt)
						db->mx_tdWT = tt;
					if (ts->tdur == 0.0)
						tt = 0.0;
					else
						tt = ts->tdUtts / ts->tdur;
					db->tdUT_num++;
					db->tdUT_sqr = db->tdUT_sqr + (tt * tt);
					db->tdUT = db->tdUT + tt;
					if (db->mn_tdUT == (float)0.0 || db->mn_tdUT > tt)
						db->mn_tdUT = tt;
					if (db->mx_tdUT < tt)
						db->mx_tdUT = tt;

					if (!isRawVal) {
						if (ts->frTokens == 0.0)
							ts->werr = 0.0;
						else
							ts->werr = (ts->werr / ts->frTokens) * 100.0000;
					}
					db->werr_num++;
					db->werr_sqr = db->werr_sqr + (ts->werr * ts->werr);
					db->werr = db->werr + ts->werr;
					if (db->mn_werr == (float)0.0 || db->mn_werr > ts->werr)
						db->mn_werr = ts->werr;
					if (db->mx_werr < ts->werr)
						db->mx_werr = ts->werr;
					db->uerr_num++;
					db->uerr_sqr = db->uerr_sqr + (ts->uerr * ts->uerr);
					db->uerr = db->uerr + ts->uerr;
					if (db->mn_uerr == (float)0.0 || db->mn_uerr > ts->uerr)
						db->mn_uerr = ts->uerr;
					if (db->mx_uerr < ts->uerr)
						db->mx_uerr = ts->uerr;
					db->retr_num++;
					db->retr_sqr = db->retr_sqr + (ts->retr * ts->retr);
					db->retr = db->retr + ts->retr;
					if (db->mn_retr == (float)0.0 || db->mn_retr > ts->retr)
						db->mn_retr = ts->retr;
					if (db->mx_retr < ts->retr)
						db->mx_retr = ts->retr;
					db->repet_num++;
					db->repet_sqr = db->repet_sqr + (ts->repet * ts->repet);
					db->repet = db->repet + ts->repet;
					if (db->mn_repet == (float)0.0 || db->mn_repet > ts->repet)
						db->mn_repet = ts->repet;
					if (db->mx_repet < ts->repet)
						db->mx_repet = ts->repet;
					if (strcmp(rulesfile, DSSNOCOMPUTE) != 0 && ts->dssUttCnt >= 50.0) {
						db->dssUttCnt_num++;
						db->dssUttCnt_sqr = db->dssUttCnt_sqr + (ts->dssUttCnt * ts->dssUttCnt);
						db->dssUttCnt = db->dssUttCnt + ts->dssUttCnt;
						if (db->mn_dssUttCnt == (float)0.0 || db->mn_dssUttCnt > ts->dssUttCnt)
							db->mn_dssUttCnt = ts->dssUttCnt;
						if (db->mx_dssUttCnt < ts->dssUttCnt)
							db->mx_dssUttCnt = ts->dssUttCnt;
						ts->dssGTotal = ts->dssGTotal / ts->dssUttCnt;
						db->dssGTotal_num++;
						db->dssGTotal_sqr = db->dssGTotal_sqr + (ts->dssGTotal * ts->dssGTotal);
						db->dssGTotal = db->dssGTotal + ts->dssGTotal;
						if (db->mn_dssGTotal == (float)0.0 || db->mn_dssGTotal > ts->dssGTotal)
							db->mn_dssGTotal = ts->dssGTotal;
						if (db->mx_dssGTotal < ts->dssGTotal)
							db->mx_dssGTotal = ts->dssGTotal;
					}
					if (ts->vocdOptimum > 0.0) {
						db->vocdOptimum_num++;
						db->vocdOptimum_sqr = db->vocdOptimum_sqr + (ts->vocdOptimum * ts->vocdOptimum);
						db->vocdOptimum = db->vocdOptimum + ts->vocdOptimum;
						if (db->mn_vocdOptimum == (float)0.0 || db->mn_vocdOptimum > ts->vocdOptimum)
							db->mn_vocdOptimum = ts->vocdOptimum;
						if (db->mx_vocdOptimum < ts->vocdOptimum)
							db->mx_vocdOptimum = ts->vocdOptimum;
					}
					if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0 && ts->ipsynUttCnt >= 100.0) {
						db->ipsynUttCnt_num++;
						db->ipsynUttCnt_sqr = db->ipsynUttCnt_sqr + (ts->ipsynUttCnt * ts->ipsynUttCnt);
						db->ipsynUttCnt = db->ipsynUttCnt + ts->ipsynUttCnt;
						if (db->mn_ipsynUttCnt == (float)0.0 || db->mn_ipsynUttCnt > ts->ipsynUttCnt)
							db->mn_ipsynUttCnt = ts->ipsynUttCnt;
						if (db->mx_ipsynUttCnt < ts->ipsynUttCnt)
							db->mx_ipsynUttCnt = ts->ipsynUttCnt;
						db->ipsynScore_num++;
						db->ipsynScore_sqr = db->ipsynScore_sqr + (ts->ipsynScore * ts->ipsynScore);
						db->ipsynScore = db->ipsynScore + ts->ipsynScore;
						if (db->mn_ipsynScore == (float)0.0 || db->mn_ipsynScore > ts->ipsynScore)
							db->mn_ipsynScore = ts->ipsynScore;
						if (db->mx_ipsynScore < ts->ipsynScore)
							db->mx_ipsynScore = ts->ipsynScore;
					}
					db->morTotal_num++;
					db->morTotal_sqr = db->morTotal_sqr + (ts->morTotal * ts->morTotal);
					db->morTotal = db->morTotal + ts->morTotal;
					if (db->mn_morTotal == (float)0.0 || db->mn_morTotal > ts->morTotal)
						db->mn_morTotal = ts->morTotal;
					if (db->mx_morTotal < ts->morTotal)
						db->mx_morTotal = ts->morTotal;
					
					db->morItems_num++;
					for (col=labelsRoot; col != NULL; col=col->next_label) {
						tt = ts->mor_count[col->num];
						db->morItems_sqr[col->num] = db->morItems_sqr[col->num] + (tt * tt);
						db->morItems[col->num] = db->morItems[col->num] + tt;
					}
				}

				db->db_sp = freespeakers(db->db_sp);

			}
			ts = NULL;
			if (isNOptionSet == FALSE) {
				strcpy(kideval_BBS, "@*&#");
				strcpy(kideval_CBS, "@*&#");
			}
			if (kideval_SpecWords) {
				isOutputGem = FALSE;
			} else {
				isOutputGem = TRUE;
			}
			rightIDFound = FALSE;
			found = 0;
			IDText[0] = EOS;
			word[0] = EOS;
			DBFname[0] = EOS;
		} else {
			if (templineC[0] == 'G') {
				strcpy(utterance->speaker, "@G:");
				strcpy(utterance->line, templineC+1);
			} else if (templineC[0] == 'B') {
				strcpy(utterance->speaker, "@BG:");
				strcpy(utterance->line, templineC+1);
			} else if (templineC[0] == 'E') {
				strcpy(utterance->speaker, "@EG:");
				strcpy(utterance->line, templineC+1);
			} else if (isdigit(templineC[0])) {
				strcpy(utterance->speaker, "*CHI:");
				strcpy(utterance->line, templineC);
			}
			if (uttline != utterance->line)
				strcpy(uttline,utterance->line);
			if (kideval_SpecWords && !strcmp(kideval_BBS, "@*&#") && rightIDFound) {
				if (uS.partcmp(utterance->speaker,"@BG:",FALSE,FALSE)) {
					kideval_n_option = FALSE;
					strcpy(kideval_BBS, "@BG:");
					strcpy(kideval_CBS, "@EG:");
				} else if (uS.partcmp(utterance->speaker,"@G:",FALSE,FALSE)) {
					kideval_n_option = TRUE;
					strcpy(kideval_BBS, "@G:");
					strcpy(kideval_CBS, "@*&#");
				}
			}
			if (uS.partcmp(utterance->speaker,kideval_BBS,FALSE,FALSE)) {
				if (kideval_n_option) {
					if (isRightText(word)) {
						isOutputGem = TRUE;
					} else
						isOutputGem = FALSE;
				} else {
					if (isRightText(word)) {
						found++;
						if (found == 1 || GemMode != '\0') {
							isOutputGem = TRUE;
						}
					}
				}
			} else if (found > 0 && uS.partcmp(utterance->speaker,kideval_CBS,FALSE,FALSE)) {
				if (kideval_n_option) {
				} else {
					if (isRightText(word)) {
						found--;
						if (found == 0) {
							if (kideval_SpecWords)
								isOutputGem = FALSE;
							else {
								isOutputGem = TRUE;
							}
						}
					}
				}
			} else if (isOutputGem) {
				if (utterance->speaker[0] == '*') {
					uS.remFrontAndBackBlanks(utterance->line);
					strcpy(templineC, utterance->speaker);
					ts = kideval_FindSpeaker(FileName1, templineC, NULL, TRUE, db);
					if (ts != NULL)
						retrieveTS(ts, utterance->line, db);
				} else
					kideval_process_tier(ts, db, NULL, IDText);
			}
		}
	}
	fpin = tfpin;
}

static void ParseDatabase(struct database *db, char *lang) {
	char   isDbSpFound = FALSE;
	FILE *fp;

	// remove beg
/*
#ifdef UNX
	strcpy(FileName1, lib_dir);
#else
	strcpy(FileName1, prefsDir);
#endif
	addFilename2Path(FileName1, lang);
	strcat(FileName1, DB_type);
	strcat(FileName1, DATABASE_FILE_NAME);
	fp = fopen(FileName1, "r");
	if (fp == NULL) {
*/
		strcpy(FileName1, lib_dir);
		addFilename2Path(FileName1, "kideval");
		addFilename2Path(FileName1, lang);
		strcat(FileName1, DB_type);
		strcat(FileName1, DATABASE_FILE_NAME);
		fp = fopen(FileName1, "r");
//	}
	if (fp == NULL) {
		fprintf(stderr, "\nCan't find database in libary \"%s\"\n", FileName1);
#ifdef UNX
		fprintf(stderr,"Please use -L option to specify libary path.\n");
#else
		fprintf(stderr,"Please make sure that the \"lib\" in the Commands window is set to the \"clan/lib\" directory.\n");
#endif
//		fprintf(stderr, "\t or in preferences folder \n");
//		fprintf(stderr, "Please download database with \"File->Get KIDEVAL Database\" menu\n");
		fprintf(stderr, "If you do not use database, then do not include +d option in command line\n");
		kideval_error(db, FALSE);
	} else {
		fprintf(stderr, "    Database File used: %s\n", FileName1);
		OpenDBFile(fp, db, &isDbSpFound);
//		fprintf(stderr, "    %.0f      \n", db->tUtt_num);
	}
	if (!isDbSpFound || db->tUtt_num == 0.0) {
		if (kideval_SpecWords) {
			fprintf(stderr,"\nERROR: No speaker matching +d option found in the database\n");
			fprintf(stderr,"OR No specified Gems found for selected speakers in the database\n\n");
		} else
			fprintf(stderr,"\nERROR: No speaker matching +d option found in the database\n\n");
		kideval_error(db, FALSE);
	}	
	SetNewVol(wd_dir);
}

static void compute_SD(struct SDs *SD, float score, float *div, float sqr_mean, float mean, float num) {
	float cSD;

	if (!SD->isSDComputed) {
		if (num < 2.0) {
			SD->dbSD = 0.0;
			SD->isSDComputed = 2;
		} else {
			SD->dbSD = (sqr_mean - ((mean * mean) / num)) / (num - 1);
			if (SD->dbSD < 0.0)
				SD->dbSD = -SD->dbSD;
			SD->dbSD = sqrt(SD->dbSD);
			SD->isSDComputed = 1;
		}
	}
	if (SD->dbSD == 0.0 || (div != NULL && *div <= 0.0)) {
		excelStrCell(fpout, "N/A");
		SD->stars = 0;
	} else {
		if (div != NULL && *div != 0.0)
			score = score / (*div);
		mean = mean / num; 
		cSD = (score - mean) / SD->dbSD;
		excelNumCell(fpout, "%.3f", cSD);
		if (cSD <= -2.0000 || cSD >= 2.0000)
			SD->stars = 2;
		else if (cSD <= -1.0000 || cSD >= 1.0000)
			SD->stars = 1;
		else
			SD->stars = 0;
	}
}

static void set_SD2NA(struct SDs *SD, float sqr_mean, float mean, float num) {
	if (!SD->isSDComputed) {
		if (num < 2.0) {
			SD->dbSD = 0.0;
			SD->isSDComputed = 2;
		} else {
			SD->dbSD = (sqr_mean - ((mean * mean) / num)) / (num - 1);
			if (SD->dbSD < 0.0)
				SD->dbSD = -SD->dbSD;
			SD->dbSD = sqrt(SD->dbSD);
			SD->isSDComputed = 1;
		}
	}
	excelStrCell(fpout, "N/A");
	SD->stars = 0;
}

static char *getFileName(char *oFname) {
	int  i, cnt;

	cnt = 0;
	i=strlen(oFname) - 1;
	while (i >= 0) {
		if (oFname[i] == PATHDELIMCHR) {
			cnt++;
			if (cnt == 2)
				break;
		}
		i--;
	}
	i++;
	return(oFname+i);
}

static void kideval_pr_result(void) {
	int    i, SDn = 0;
	char   *sFName;
	float  tNum;
	struct kideval_speakers *ts;
	struct SDs SD[SDRESSIZE];
	struct DBKeys *DBKey;
	struct label_cols *col;

	if (isCreateDB) {
		sp_head = freespeakers(sp_head);
		return;
	}
	if (sp_head == NULL) {
		if (kideval_SpecWords && !onlyApplyToDB) {
			fprintf(stderr,"\nERROR: No speaker matching +t option found\n");
			fprintf(stderr,"OR No specified Gems found for this speaker\n");
		} else
			fprintf(stderr,"\nERROR: No speaker matching +t option found\n");
		if (isLinkAge)
			fprintf(stderr,"OR No input file had speaker in the right age range\n");
		fprintf(stderr,"\n");
	}
	excelHeader(fpout, newfname, 95);
	excelRow(fpout, ExcelRowStart);
	if (DBKeyRoot != NULL) {
		for (i=0; i < SDRESSIZE; i++) {
			SD[i].isSDComputed = FALSE;
		}
		excelStrCell(fpout,"File/DB");
	} else
		excelStrCell(fpout,"File");
	excelCommasStrCell(fpout, "Language,Corpus,Code,Age(Month),Sex,Group,SES,Role,Education,Custom_field");
	excelCommasStrCell(fpout, "Total Utts");
	excelCommasStrCell(fpout, "MLU Utts,MLU Words,MLU Morphemes,MLU100 Utts,MLU100 Words,MLU100 Morphemes");
	excelCommasStrCell(fpout, "FREQ types,FREQ tokens,FREQ TTR");
	excelCommasStrCell(fpout, "NDW/100");
	excelCommasStrCell(fpout, "VOCD D_optimum_average");
	excelCommasStrCell(fpout, "Verbs/Utt");
	if (DBKeyRoot == NULL) { // exclude TD columns
		excelCommasStrCell(fpout, "TD Words,TD Utts,TD Time (secs),TD Words/Time,TD Utts/Time");
	}
	if (isRawVal) {
		excelCommasStrCell(fpout, "Word Errors");
	} else {
		excelCommasStrCell(fpout, "%% Word Errors");
	}
	excelCommasStrCell(fpout, "Utt Errors");
	excelCommasStrCell(fpout, "retracing[//],repetition[/]");
	if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
		excelCommasStrCell(fpout, "DSS Utts,DSS");
	}
	if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
		excelCommasStrCell(fpout, "IPSyn Utts,IPSyn Total");
	}
	excelCommasStrCell(fpout, "mor Words");
	for (col=labelsRoot; col != NULL; col=col->next_label) {
		if (col->label != NULL)
			excelStrCell(fpout, col->label);
	}
	excelRow(fpout, ExcelRowEnd);
	for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
		if (!ts->isSpeakerFound) {
			if (kideval_SpecWords) {
				fprintf(stderr,"\nWARNING: No specified Gems found for speaker \"%s\" in file \"%s\"\n\n", ts->sp, ts->fname);
			} else
				fprintf(stderr,"\nWARNING: No data found for speaker \"%s\" in file \"%s\"\n\n", ts->sp, ts->fname);
			continue;
		}
		if (ts->tUtt < MinUttLimit)
			continue;
// ".xls"
		excelRow(fpout, ExcelRowStart);
		sFName = getFileName(ts->fname);
		excelStrCell(fpout, sFName);
		if (ts->ID) {
			excelOutputID(fpout, ts->ID);
		} else {
			excelCommasStrCell(fpout, ".,.");
			excelStrCell(fpout, ts->sp);
			excelCommasStrCell(fpout, ".,.,.,.,.,.,.");
		}
		if (!ts->isMORFound) {
			fprintf(stderr,"\n*** File \"%s\": Speaker \"%s\"\n", sFName, ts->sp);
			fprintf(stderr,"WARNING: Speaker \"%s\" has no \"%s\" tiers.\n\n", ts->sp, fDepTierName);
			excelStrCell(fpout, "0");
			excelCommasStrCell(fpout, "0,0,0,0,0,0");
			excelCommasStrCell(fpout, "0,0,0");
			excelStrCell(fpout, "0");
			excelStrCell(fpout, "0");
			excelStrCell(fpout, "0");
			if (DBKeyRoot == NULL) // exclude TD columns
				excelCommasStrCell(fpout, "0,0,0,0,0");
			excelStrCell(fpout, "0");
			excelStrCell(fpout, "0");
			excelCommasStrCell(fpout, "0,0");
			if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
				excelCommasStrCell(fpout, "0,0");
			}
			if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
				excelCommasStrCell(fpout, "0,0");
			}
			excelStrCell(fpout, "0");
			for (col=labelsRoot; col != NULL; col=col->next_label) {
				excelStrCell(fpout, "0");
			}
			excelRow(fpout, ExcelRowEnd);
			excelRow(fpout, ExcelRowEmpty);
			continue;
		}
		excelNumCell(fpout, "%.0f", ts->tUtt);
		excelNumCell(fpout, "%.0f", ts->mUtt);
		if (ts->mUtt == 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->mWords/ts->mUtt);
		if (ts->mUtt == 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->morf/ts->mUtt);
		excelNumCell(fpout, "%.0f", ts->mUtt100);
		if (ts->mUtt100 < 100.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->mWords100/ts->mUtt100);
		if (ts->mUtt100 < 100.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->morf100/ts->mUtt100);
		excelNumCell(fpout, "%.0f", ts->frTypes);
		excelNumCell(fpout, "%.0f", ts->frTokens);
		if (ts->frTokens == 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->frTypes/ts->frTokens);
		if (ts->NDWTotal < 100)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.0f", ts->NDW);
		if (ts->vocdOptimum <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.2f", ts->vocdOptimum);
		if (ts->tUtt == 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f", ts->CUR/ts->tUtt);
		if (DBKeyRoot == NULL) { // exclude TD columns
			excelNumCell(fpout, "%.0f", ts->tdWords);
			excelNumCell(fpout, "%.0f", ts->tdUtts);
			excelNumCell(fpout, "%.0f", ts->tdur);
			if (ts->tdur == 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f", ts->tdWords/ts->tdur);
			if (ts->tdur == 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f", ts->tdUtts/ts->tdur);
		}
		if (!isRawVal) {
			if (ts->frTokens == 0.0)
				ts->werr = 0.000;
			else
				ts->werr = (ts->werr / ts->frTokens) * 100.0000;
			excelNumCell(fpout, "%.3f", ts->werr);
		} else
			excelNumCell(fpout, "%.0f", ts->werr);
		excelNumCell(fpout, "%.0f", ts->uerr);
		excelNumCell(fpout, "%.0f", ts->retr);
		excelNumCell(fpout, "%.0f", ts->repet);
		if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
			excelNumCell(fpout, "%.0f", ts->dssUttCnt);
			tNum = ts->dssGTotal;
			if (DssIpsynUttLimit > 0 && ts->dssUttCnt <= DssIpsynUttLimit)
				excelNumCell(fpout, "%.2f", tNum / ts->dssUttCnt);
			else if (ts->dssUttCnt < 50.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.2f", tNum / ts->dssUttCnt);
		}
		if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
			excelNumCell(fpout, "%.0f", ts->ipsynUttCnt);
			if (DssIpsynUttLimit > 0 && ts->dssUttCnt <= DssIpsynUttLimit)
				excelNumCell(fpout, "%.2f", ts->ipsynScore);
			else if (ts->ipsynUttCnt < 100.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.2f", ts->ipsynScore);
		}
		excelNumCell(fpout, "%.0f", ts->morTotal);
		for (col=labelsRoot; col != NULL; col=col->next_label) {
			if (ts->mor_count == NULL) {
				excelStrCell(fpout, ".");
			} else {
				if (!isRawVal) {
					tNum = ts->mor_count[col->num];
					if (ts->morTotal == 0.0)
						excelStrCell(fpout, "0.000");
					else
						excelNumCell(fpout, "%.3f", (tNum/ts->morTotal)*100.0000);
				} else
					excelNumCell(fpout, "%.0f", ts->mor_count[col->num]);
			}
		}
		excelRow(fpout, ExcelRowEnd);
		if (DBKeyRoot != NULL) {
			excelRow(fpout, ExcelRowStart);
			excelCommasStrCell(fpout,"+/-SD,.,.,.,.,.,.,.,.,.,.");

			SDn = 0; // this number should be less than SDRESSIZE = 256
			compute_SD(&SD[SDn++], ts->tUtt,  NULL, gdb->tUtt_sqr, gdb->tUtt, gdb->tUtt_num);
			compute_SD(&SD[SDn++], ts->mUtt,  NULL, gdb->mUtt_sqr, gdb->mUtt, gdb->mUtt_num);

			compute_SD(&SD[SDn++], ts->mWords,  &ts->mUtt, gdb->mWords_sqr, gdb->mWords, gdb->mWords_num);
			compute_SD(&SD[SDn++], ts->morf,  &ts->mUtt, gdb->morf_sqr, gdb->morf, gdb->morf_num);

// for mUtt100 always say N/A
			excelStrCell(fpout, "N/A");
			SD[SDn++].stars = 0;
//			compute_SD(&SD[SDn++], ts->mUtt100,  NULL, gdb->mUtt100_sqr, gdb->mUtt100, gdb->mUtt100_num);
			if (ts->mUtt100 < 100.0) {
				set_SD2NA(&SD[SDn++], gdb->mWords100_sqr, gdb->mWords100, gdb->mWords100_num);
				set_SD2NA(&SD[SDn++], gdb->morf100_sqr, gdb->morf100, gdb->morf100_num);
			} else {
				compute_SD(&SD[SDn++], ts->mWords100,  &ts->mUtt100, gdb->mWords100_sqr, gdb->mWords100, gdb->mWords100_num);
				compute_SD(&SD[SDn++], ts->morf100,  &ts->mUtt100, gdb->morf100_sqr, gdb->morf100, gdb->morf100_num);
			}
			compute_SD(&SD[SDn++], ts->frTypes,  NULL, gdb->frTypes_sqr, gdb->frTypes, gdb->frTypes_num);
			compute_SD(&SD[SDn++], ts->frTokens,  NULL, gdb->frTokens_sqr, gdb->frTokens, gdb->frTokens_num);
			compute_SD(&SD[SDn++], ts->frTypes,  &ts->frTokens, gdb->TTR_sqr, gdb->TTR, gdb->TTR_num);
			if (ts->NDWTotal < 100)
				set_SD2NA(&SD[SDn++], gdb->NDW_sqr, gdb->NDW, gdb->NDW_num);
			else
				compute_SD(&SD[SDn++], ts->NDW,  NULL, gdb->NDW_sqr, gdb->NDW, gdb->NDW_num);

			if (ts->vocdOptimum <= 0.0)
				set_SD2NA(&SD[SDn++], gdb->vocdOptimum_sqr, gdb->vocdOptimum, gdb->vocdOptimum_num);
			else
				compute_SD(&SD[SDn++], ts->vocdOptimum,  NULL, gdb->vocdOptimum_sqr, gdb->vocdOptimum, gdb->vocdOptimum_num);
			compute_SD(&SD[SDn++], ts->CUR,  &ts->tUtt, gdb->CUR_sqr, gdb->CUR, gdb->CUR_num);
/* exclude TD columns
			compute_SD(&SD[SDn++], ts->tdWords,  NULL, gdb->tdWords_sqr, gdb->tdWords, gdb->tdWords_num);
			compute_SD(&SD[SDn++], ts->tdUtts,  NULL, gdb->tdUtts_sqr, gdb->tdUtts, gdb->tdUtts_num);
			compute_SD(&SD[SDn++], ts->tdur,  NULL, gdb->tdur_sqr, gdb->tdur, gdb->tdur_num);
			compute_SD(&SD[SDn++], ts->tdWords,  &ts->tdur, gdb->tdWT_sqr, gdb->tdWT, gdb->tdWT_num);
			compute_SD(&SD[SDn++], ts->tdUtts,  &ts->tdur, gdb->tdUT_sqr, gdb->tdUT, gdb->tdUT_num);
*/
			compute_SD(&SD[SDn++], ts->werr,  NULL, gdb->werr_sqr, gdb->werr, gdb->werr_num);
			compute_SD(&SD[SDn++], ts->uerr,  NULL, gdb->uerr_sqr, gdb->uerr, gdb->uerr_num);

			compute_SD(&SD[SDn++], ts->retr,  NULL, gdb->retr_sqr, gdb->retr, gdb->retr_num);
			compute_SD(&SD[SDn++], ts->repet,  NULL, gdb->repet_sqr, gdb->repet, gdb->repet_num);

			if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
// for DSS Utts always say N/A
				excelStrCell(fpout, "N/A");
				SD[SDn++].stars = 0;
//				compute_SD(&SD[SDn++], ts->dssUttCnt,  NULL, gdb->dssUttCnt_sqr, gdb->dssUttCnt, gdb->dssUttCnt_num);
				compute_SD(&SD[SDn++], ts->dssGTotal,  &ts->dssUttCnt, gdb->dssGTotal_sqr, gdb->dssGTotal, gdb->dssGTotal_num);
			}

			if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
// for IPSyn Utts always say N/A
				excelStrCell(fpout, "N/A");
				SD[SDn++].stars = 0;
//				compute_SD(&SD[SDn++], ts->ipsynUttCnt,  NULL, gdb->ipsynUttCnt_sqr, gdb->ipsynUttCnt, gdb->ipsynUttCnt_num);
				if (ts->ipsynScore <= 0.0)
					set_SD2NA(&SD[SDn++], gdb->ipsynScore_sqr, gdb->ipsynScore, gdb->ipsynScore_num);
				else
					compute_SD(&SD[SDn++], ts->ipsynScore,  NULL, gdb->ipsynScore_sqr, gdb->ipsynScore, gdb->ipsynScore_num);
			}
			compute_SD(&SD[SDn++], ts->morTotal,  NULL, gdb->morTotal_sqr, gdb->morTotal, gdb->morTotal_num);

			for (col=labelsRoot; col != NULL; col=col->next_label) {
				if (ts->mor_count != NULL) {
					compute_SD(&SD[SDn++], ts->mor_count[col->num],  NULL, gdb->morItems_sqr[col->num], gdb->morItems[col->num], gdb->morItems_num);
				}
			}
			excelRow(fpout, ExcelRowEnd);

			excelRow(fpout, ExcelRowStart);
			excelCommasStrCell(fpout, " , , , , , , , , , , ");
			for (i=0; i < SDn; i++) {
				if (SD[i].stars >= 2)
					excelStrCell(fpout, "**");
				else if (SD[i].stars >= 1)
					excelStrCell(fpout, "*");
				else
					excelStrCell(fpout, " ");
			}
			excelRow(fpout, ExcelRowEnd);
		}
	}
	if (DBKeyRoot != NULL) {
		excelRow(fpout, ExcelRowStart);
		excelCommasStrCell(fpout,"Mean Database,.,.,.,.,.,.,.,.,.,.");
		if (gdb->tUtt_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->tUtt/gdb->tUtt_num);
		if (gdb->mUtt_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->mUtt/gdb->mUtt_num);
		if (gdb->mWords_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->mWords/gdb->mWords_num);
		if (gdb->morf_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->morf/gdb->morf_num);

		if (gdb->mUtt100_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->mUtt100/gdb->mUtt100_num);
		if (gdb->mWords100_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->mWords100/gdb->mWords100_num);
		if (gdb->morf100_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->morf100/gdb->morf100_num);

		if (gdb->frTypes_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->frTypes/gdb->frTypes_num);
		if (gdb->frTokens_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->frTokens/gdb->frTokens_num);
		if (gdb->TTR_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->TTR/gdb->TTR_num);
		if (gdb->NDW_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->NDW/gdb->NDW_num);

		if (gdb->vocdOptimum_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->vocdOptimum/gdb->vocdOptimum_num);
		if (gdb->CUR_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->CUR/gdb->CUR_num);
/* exclude TD columns
		if (gdb->tdWords_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->tdWords/gdb->tdWords_num);
		if (gdb->tdUtts_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->tdUtts/gdb->tdUtts_num);
		if (gdb->tdur_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.0f",gdb->tdur/gdb->tdur_num);
		if (gdb->tdWT_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->tdWT/gdb->tdWT_num);
		if (gdb->tdUT_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->tdUT/gdb->tdUT_num);
*/
		if (gdb->werr_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->werr/gdb->werr_num);
		if (gdb->uerr_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->uerr/gdb->uerr_num);

		if (gdb->retr_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->retr/gdb->retr_num);
		if (gdb->repet_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->repet/gdb->repet_num);

		if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
			if (gdb->dssUttCnt_num <= 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f",gdb->dssUttCnt/gdb->dssUttCnt_num);
			if (gdb->dssGTotal_num <= 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f",gdb->dssGTotal/gdb->dssGTotal_num);
		}

		if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
			if (gdb->ipsynUttCnt_num <= 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f",gdb->ipsynUttCnt/gdb->ipsynUttCnt_num);
			if (gdb->ipsynScore_num <= 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f",gdb->ipsynScore/gdb->ipsynScore_num);
		}

		if (gdb->morTotal_num <= 0.0)
			excelStrCell(fpout, "N/A");
		else
			excelNumCell(fpout, "%.3f",gdb->morTotal/gdb->morTotal_num);

		for (col=labelsRoot; col != NULL; col=col->next_label) {
			if (gdb->morItems_num <= 0.0)
				excelStrCell(fpout, "N/A");
			else
				excelNumCell(fpout, "%.3f",gdb->morItems[col->num]/gdb->morItems_num);
		}
		excelRow(fpout, ExcelRowEnd);

		excelRow(fpout, ExcelRowStart);
		excelCommasStrCell(fpout,"SD Database,.,.,.,.,.,.,.,.,.,.");
		for (i=0; i < SDn; i++) {
			if (SD[i].isSDComputed == 1) {
				excelNumCell(fpout, "%.3f", SD[i].dbSD);
			} else {
				excelStrCell(fpout, "N/A");
			}
		}
		excelRow(fpout, ExcelRowEnd);

		excelRow(fpout, ExcelRowStart);
		excelCommasStrCell(fpout,"Number files,.,.,.,.,.,.,.,.,.,.");
		excelNumCell(fpout, "%.0f",gdb->tUtt_num);
		excelNumCell(fpout, "%.0f",gdb->mUtt_num);

		excelNumCell(fpout, "%.0f",gdb->mWords_num);
		excelNumCell(fpout, "%.0f",gdb->morf_num);

		excelNumCell(fpout, "%.0f",gdb->mUtt100_num);
		excelNumCell(fpout, "%.0f",gdb->mWords100_num);
		excelNumCell(fpout, "%.0f",gdb->morf100_num);

		excelNumCell(fpout, "%.0f",gdb->frTypes_num);
		excelNumCell(fpout, "%.0f",gdb->frTokens_num);
		excelNumCell(fpout, "%.0f",gdb->TTR_num);
		excelNumCell(fpout, "%.0f",gdb->NDW_num);

		excelNumCell(fpout, "%.0f",gdb->vocdOptimum_num);
		excelNumCell(fpout, "%.0f",gdb->CUR_num);
/* exclude TD columns
		excelNumCell(fpout, "%.0f",gdb->tdWords_num);
		excelNumCell(fpout, "%.0f",gdb->tdUtts_num);
		excelNumCell(fpout, "%.0f",gdb->tdur_num);
		excelNumCell(fpout, "%.0f",gdb->tdWT_num);
		excelNumCell(fpout, "%.0f",gdb->tdUT_num);
*/
		excelNumCell(fpout, "%.0f",gdb->werr_num);
		excelNumCell(fpout, "%.0f",gdb->uerr_num);

		excelNumCell(fpout, "%.0f",gdb->retr_num);
		excelNumCell(fpout, "%.0f",gdb->repet_num);

		if (strcmp(rulesfile, DSSNOCOMPUTE) != 0) {
			excelNumCell(fpout, "%.0f",gdb->dssUttCnt_num);
			excelNumCell(fpout, "%.0f",gdb->dssGTotal_num);
		}
		if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
			excelNumCell(fpout, "%.0f",gdb->ipsynUttCnt_num);
			excelNumCell(fpout, "%.0f",gdb->ipsynScore_num);
		}
		excelNumCell(fpout, "%.0f",gdb->morTotal_num);

		for (col=labelsRoot; col != NULL; col=col->next_label) {
			if (gdb->morItems != NULL) {
				excelNumCell(fpout, "%.0f",gdb->morItems_num);
			}
		}
		excelRow(fpout, ExcelRowEnd);
	}
	excelRow(fpout, ExcelRowEmpty);
	if (sp_head != NULL) {
		if (DBKeyRoot != NULL) {
			excelRowOneStrCell(fpout, ExcelRedCell, "CURRENT COMPARISONS ARE PRELIMINARY AND SHOULD NOT BE USED FOR CLINICAL DIAGNOSIS.");
			excelRowOneStrCell(fpout, ExcelRedCell, "PLEASE SEE SLP MANUAL FOR RECOMMENDED INTERPRETATION.");
			excelRow(fpout, ExcelRowEmpty);
			excelRowOneStrCell(fpout, ExcelBlkCell, "+/- SD  * = 1 SD, ** = 2 SD");
		}
		if (DBGemNum) {
			strcpy(templineC4, "Database Gems:");
			for (i=0; i < DBGemNum; i++) {
				strcat(templineC4, DBGems[i]);
				strcat(templineC4, ", ");
			}
			uS.remblanks(templineC4);
			i = strlen(templineC4) - 1;
			if (templineC4[i] == ',')
				templineC4[i] = EOS;
			excelRowOneStrCell(fpout, ExcelBlkCell, templineC4);
		}
		if (DBKeyRoot != NULL) {
			strcpy(templineC4, "Database keywords: ");
			i = strlen(templineC4);
			for (DBKey=DBKeyRoot; DBKey != NULL; DBKey=DBKey->next) {
				if (DBKey->key1 != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->key1);
				}
				if (DBKey->key2 != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->key2);
				}
				if (DBKey->key3 != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->key3);
				}
				if (DBKey->key4 != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->key4);
				}
				if (DBKey->age != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->age);
				}
				if (DBKey->sex != NULL) {
					if (templineC4[i] != EOS)
						strcat(templineC4, "|");
					strcat(templineC4, DBKey->sex);
				}
				strcat(templineC4, ", ");
			}
			uS.remblanks(templineC4);
			i = strlen(templineC4) - 1;
			if (templineC4[i] == ',')
				templineC4[i] = EOS;
			excelRowOneStrCell(fpout, ExcelBlkCell, templineC4);

			strcpy(templineC4, "Database date: ");
			strcat(templineC4, DB_version);
			excelRowOneStrCell(fpout, ExcelBlkCell, templineC4);
		}
//		printArg(targv, targc, fpout, FALSE, "");
	}
	excelFooter(fpout);
	sp_head = freespeakers(sp_head);
}

static char isFoundGems(void) {
	while (fgets_cr(templineC, UTTLINELEN, fpin)) {
		if (uS.partcmp(templineC,"@BG:",FALSE,FALSE) ||
			uS.partcmp(templineC,"@G:",FALSE,FALSE)) {
			rewind(fpin);
			return(TRUE);
		}
	}
	rewind(fpin);
	return(FALSE);
}

static void init_words(int command) {
	if (command == BASE_COM) {
		addword('\0','\0',"+</>");
		addword('\0','\0',"+<//>");
		addword('\0','\0',"+<///>");
		addword('\0','\0',"+</->");
		addword('\0','\0',"+</?>");
		addword('\0','\0',"+0*");
		addword('\0','\0',"+&*");
		addword('\0','\0',"+-*");
		addword('\0','\0',"+#*");
		addword('\0','\0',"+(*.*)");
//		addword('\0','\0',"+xxx");
//		addword('\0','\0',"+yyy");
//		addword('\0','\0',"+www");
//		addword('\0','\0',"+xxx@s*");
//		addword('\0','\0',"+yyy@s*");
//		addword('\0','\0',"+www@s*");
		addword('\0','\0',"+unk|xxx");
		addword('\0','\0',"+unk|yyy");
		addword('\0','\0',"+*|www");
		mor_initwords();
	} else if (command == DSS_COM) {
		addword('\0','\0',"+<#>");
		addword('\0','\0',"+</>");  // list R6 in mmaininit funtion in cutt.c
		addword('\0','\0',"+<//>");
		addword('\0','\0',"+<///>");
		addword('\0','\0',"+</->");
		addword('\0','\0',"+</?>");
		addword('\0','\0',"+xxx");
		addword('\0','\0',"+yyy");
		addword('\0','\0',"+www");
		addword('\0','\0',"+xxx@s*");
		addword('\0','\0',"+yyy@s*");
		addword('\0','\0',"+www@s*");
		addword('\0','\0',"+unk|xxx");
		addword('\0','\0',"+unk|yyy");
		addword('\0','\0',"+*|www");
		maininitwords();
		mor_initwords();
	} else if (command == VOCD_COM) {
		addword('\0','\0',"+xx");
		addword('\0','\0',"+xxx");
		addword('\0','\0',"+yy");
		addword('\0','\0',"+yyy");
		addword('\0','\0',"+www");
		addword('\0','\0',"+*|xx");
		addword('\0','\0',"+unk|xxx");
		addword('\0','\0',"+*|yy");
		addword('\0','\0',"+unk|yyy");
		addword('\0','\0',"+*|www");
		addword('\0','\0',"+.");
		addword('\0','\0',"+?");
		addword('\0','\0',"+!");
		maininitwords();
		mor_initwords();
	} else if (command == IPSYN_COM) {
		addword('\0','\0',"+<#>");
		addword('\0','\0',"+</>");  // list R6 in mmaininit funtion in cutt.c
		addword('\0','\0',"+<//>");
		addword('\0','\0',"+<///>");
		addword('\0','\0',"+</->");
		addword('\0','\0',"+</?>");
		addword('\0','\0',"+xxx");
		addword('\0','\0',"+yyy");
		addword('\0','\0',"+www");
		addword('\0','\0',"+xxx@s*");
		addword('\0','\0',"+yyy@s*");
		addword('\0','\0',"+www@s*");
		addword('\0','\0',"+unk|xxx");
		addword('\0','\0',"+unk|yyy");
		addword('\0','\0',"+*|www");
		maininitwords();
//		mor_initwords();
	}
}

static void init_s_option(int command) {
	int  i;

	clean_s_option();
	init_words(command);
	for (i=1; i < targc; i++) {
		if (*targv[i] == '+' || *targv[i] == '-') {
			if (targv[i][1] == 's') {
				if (i+1 < targc) {
					getflag(targv[i],targv[i+1],&i);
				} else {
					getflag(targv[i],NULL,&i);
				}
			}
		}
	}	
}

static char isExcludeUtt(char *sp, char *line, char *word) {
	int i;

	i = 0;
	while ((i=getword(sp, line, word, NULL, i))) {
		if (word[0] == '[' && word[1] == '+') {
			removeExtraSpace(word);
			if (uS.mStricmp(word, "[+ exc]") == 0)
				return(TRUE);
		} else if (!uS.mStricmp(word, "www"))
			return(TRUE);
	}
	return(FALSE);
}

static void computeDSS_VOCD_IPSYN(int misalignmentError) {
	struct kideval_speakers *ts;

	gMisalignmentError += misalignmentError;
	if (!vocd_analize(FALSE)) {
		kideval_error(NULL, FALSE);
	}
	if (misalignmentError == 0) {
		if (!compute_ipsyn(FALSE)) {
			kideval_error(NULL, FALSE);
		}
	} else {
		if (mor_link.error_found) {
			mor_link.error_found = FALSE;
		}
	}
	for (ts=sp_head; ts != NULL; ts=ts->next_sp) {
		if (uS.mStricmp(ts->fname, oldfname) == 0) {
			if (ts->dss_sp != NULL) {
				ts->dssUttCnt = ts->dss_sp->TotalNum;
				ts->dssGTotal = ts->dss_sp->GrandTotal;
				ts->dss_sp = NULL;
			} else {
				ts->dssUttCnt = 0.0;
				ts->dssGTotal = 0.0;
			}
			if (ts->vocd_sp != NULL) {
				ts->vocdOptimum = ts->vocd_sp->d_optimum;
				ts->vocd_sp = NULL;
			} else
				ts->vocdOptimum = 0.0;
			if (misalignmentError == 0 && ts->ipsyn_sp != NULL) {
				ts->ipsynUttCnt = (float)ts->ipsyn_sp->UttNum;
				ts->ipsynScore = ts->ipsyn_sp->TotalScore;
				ts->ipsyn_sp = NULL;
			} else {
				ts->ipsynUttCnt = 0.0;
				ts->ipsynScore = 0.0;
			}
		}
	}
	dss_freeSpeakers();
	ipsyn_freeSpeakers();
}

void call() {
	int  i, found, command, misalignmentError = 0;
//	long tlineno = 0;
	char isPRCreateDBRes;
	char lRightspeaker, rightIDFound;
	char isOutputGem, isGRA;
	char word[BUFSIZ+1], IDText[BUFSIZ+1], *s;
	time_t timer;
	struct tm *TmS;
	struct kideval_speakers *ts;
	struct IDparts IDTier;
	DSSSP *lastdsp;
	IPSYN_SP *lastipsp;
	extern int getc_cr_lc;
	extern char fgets_cr_lc, Tspchanged, contSpeaker[];

	IDText[0] = EOS;
	ts = NULL;
	if (isGOptionSet == FALSE)
		onlyApplyToDB = !isFoundGems();
	clean_speakers(speakers, TRUE);
	if (isCreateDB) {
		if (dbfpout == NULL) {
			strcpy(FileName1, wd_dir);
			i = strlen(FileName1) - 1;
			if (FileName1[i] == PATHDELIMCHR)
				FileName1[i] = EOS;
			s = strrchr(FileName1, PATHDELIMCHR);
			if (s != NULL) {
				*s = EOS;
			}
			addFilename2Path(FileName1, script_file);
			strcat(FileName1, DB_type);
			strcat(FileName1, DATABASE_FILE_NAME);
			dbfpout = fopen(FileName1, "wb");
			if (dbfpout == NULL) {
				fprintf(stderr,"Can't create file \"%s\", perhaps path doesn't exist or it is opened by another application\n",FileName1);
				kideval_error(NULL, FALSE);
			}
			if (dbfpout != NULL) {
				time(&timer);
				TmS = localtime(&timer);
				fprintf(dbfpout,"V%d %d-%s%d-%s%d, %s%d:%s%d\n", KIDEVAL_DB_VERSION, TmS->tm_year+1900,
						((TmS->tm_mon+1) < 10 ? "0":""), TmS->tm_mon+1, ((TmS->tm_mday) < 10 ? "0":""), TmS->tm_mday,
						((TmS->tm_hour) < 10 ? "0":""), TmS->tm_hour, ((TmS->tm_min) < 10 ? "0":""), TmS->tm_min);
			}
		}
//		if (CRDBcnt % 10 == 0) {
#ifdef GEMS_COUNT
			fprintf(dbfpout, "=%s\n", oldfname+wdOffset);
#else
			fprintf(stdout, "%s\n", oldfname+wdOffset);
#endif
			CRDBcnt = 0;
//		}
		CRDBcnt++;
//		return;
		isOutputGem = TRUE;
		currentatt = 0;
		currentchar = (char)getc_cr(fpin, &currentatt);
		while (getwholeutter()) {
			if (utterance->speaker[0] == '*')
				break;
			if (uS.partcmp(utterance->speaker,"@Comment:",FALSE,FALSE)) {
				if (strncmp(utterance->line, "EVAL DATABASE EXCLUDE", 21) == 0) {
					fprintf(stderr,"    EXCLUDED FILE: %s\n",oldfname+wdOffset);
					return;
				}
			}
		}
		rewind(fpin);
	} else {
		fprintf(stderr,"From file <%s>\n",oldfname);
		if (isLinkAge) {
			rightIDFound = FALSE;
			currentatt = 0;
			currentchar = (char)getc_cr(fpin, &currentatt);
			while (getwholeutter()) {
				if (uS.partcmp(utterance->speaker,"@ID:",FALSE,FALSE)) {
					breakIDsIntoFields(&IDTier, utterance->line);
					spareTier1[0] = EOS;
					if (IDTier.code[0] != '*')
						strcat(spareTier1, "*");
					strcat(spareTier1, IDTier.code);
					if (checktier(spareTier1)) {
						if (isKeyMatch(&IDTier)) {
							rightIDFound = TRUE;
							break;
						}
					}
				}
				if (utterance->speaker[0] == '*')
					break;
			}
			if (!rightIDFound) {
				fprintf(stderr,"\n*** File \"%s\"\n", oldfname);
				fprintf(stderr,"\tdoes not have a speaker that matches all chosen criteria.\n");
				cutt_exit(0);
//				return;
			}
			rewind(fpin);
		}
	}
	for (command=BASE_COM; command < LAST_COMMAND; command++) {
		if (command != BASE_COM) {
			rewind(fpin);
			init_s_option(command);
			getc_cr_lc = '\0';
			fgets_cr_lc = '\0';
			contSpeaker[0] = EOS;
			Tspchanged = FALSE;
			if (lineno > -1L) {
//				tlineno = 1L;
				lineno = deflineno;
			}
		}
		if (!onlyApplyToDB && isNOptionSet == FALSE) {
			strcpy(kideval_BBS, "@*&#");
			strcpy(kideval_CBS, "@*&#");
		}
		if (isCreateDB == 0) {
			if (kideval_SpecWords && !onlyApplyToDB) {
				isOutputGem = FALSE;
			} else {
				isOutputGem = TRUE;
			}
		}
		lastdsp = NULL;
		lastipsp = NULL;
		isPRCreateDBRes = FALSE;
		spareTier1[0] = EOS;
		lRightspeaker = FALSE;
		found = 0;
		misalignmentError = 0;
		isGRA = TRUE;
		currentatt = 0;
		currentchar = (char)getc_cr(fpin, &currentatt);
		while (getwholeutter()) {
/*
			if (lineno > tlineno) {
				tlineno = lineno + 200;
			}
*/
			if (!checktier(utterance->speaker)) {
				if (*utterance->speaker == '*') {
					lRightspeaker = FALSE;
					if (command == DSS_COM) {
						cleanUttline(uttline);
						strcpy(spareTier1, uttline);
					}
				}
				continue;
			} else {
				if (*utterance->speaker == '*') {
					if (isExcludeUtt(utterance->speaker, utterance->line, word))
						lRightspeaker = FALSE;
					else
						lRightspeaker = TRUE;
				}
				if (!lRightspeaker && *utterance->speaker != '@')
					continue;
			}
			if (!onlyApplyToDB && kideval_SpecWords && !strcmp(kideval_BBS, "@*&#")) {
				if (uS.partcmp(utterance->speaker,"@BG:",FALSE,FALSE)) {
					kideval_n_option = FALSE;
					strcpy(kideval_BBS, "@BG:");
					strcpy(kideval_CBS, "@EG:");
				} else if (uS.partcmp(utterance->speaker,"@G:",FALSE,FALSE)) {
					kideval_n_option = TRUE;
					strcpy(kideval_BBS, "@G:");
					strcpy(kideval_CBS, "@*&#");
				}
			}
			if (isCreateDB && command == LAST_COMMAND-1) {
				if (utterance->speaker[0] == '@') {
					uS.remFrontAndBackBlanks(utterance->line);
					removeExtraSpace(utterance->line);
				}
#ifdef GEMS_COUNT
				if (uS.partcmp(utterance->speaker,"@G:",FALSE,FALSE)) {
					computeDSS_VOCD_IPSYN(misalignmentError);
					if (ts != NULL) {
						if (isPRCreateDBRes) {
							prTSResults(ts, IDText);
							isPRCreateDBRes = FALSE;
							misalignmentError = 0;
						}
//						if (ts->words_root != NULL)
//							ts->words_root = kideval_freetree(ts->words_root);
//						if (ts->NDW_root != NULL)
//							ts->NDW_root = kideval_freetree(ts->NDW_root);
						kideval_initTSVars(ts, FALSE);
					}
					fprintf(dbfpout, "G%s\n", utterance->line);
				} else if (uS.partcmp(utterance->speaker,"@BG:",FALSE,FALSE)) {
					fprintf(dbfpout, "B%s\n", utterance->line);
				} else if (uS.partcmp(utterance->speaker,"@EG:",FALSE,FALSE)) {
					computeDSS_VOCD_IPSYN(misalignmentError);
					if (ts != NULL) {
						if (isPRCreateDBRes) {
							prTSResults(ts, IDText);
							isPRCreateDBRes = FALSE;
							misalignmentError = 0;
						}
//						if (ts->words_root != NULL)
//							ts->words_root = kideval_freetree(ts->words_root);
//						if (ts->NDW_root != NULL)
//							ts->NDW_root = kideval_freetree(ts->NDW_root);
						kideval_initTSVars(ts, FALSE);
					}
					fprintf(dbfpout, "E%s\n", utterance->line);
				} else
#endif
				if (uS.partcmp(utterance->speaker,"@End",FALSE,FALSE)) {
					computeDSS_VOCD_IPSYN(misalignmentError);
					if (isPRCreateDBRes) {
#ifdef GEMS_COUNT
						prTSResults(ts, IDText);
#else
						if (ts->tUtt >= 50) {
							prTSResults(ts, IDText);
							fprintf(dbfpout, "-\n");
						}
#endif
						isPRCreateDBRes = FALSE;
						misalignmentError = 0;
					}
					if (ts != NULL) {
						if (ts->words_root != NULL)
							ts->words_root = kideval_freetree(ts->words_root);
						if (ts->NDW_root != NULL)
							ts->NDW_root = kideval_freetree(ts->NDW_root);
					}
#ifdef GEMS_COUNT
					fprintf(dbfpout, "-\n");
#endif
				}
			}
			if (!onlyApplyToDB && uS.partcmp(utterance->speaker,kideval_BBS,FALSE,FALSE)) {
				if (kideval_n_option) {
					if (isRightText(word)) {
						isOutputGem = TRUE;
					} else
						isOutputGem = FALSE;
				} else {
					if (isRightText(word)) {
						found++;
						if (found == 1 || GemMode != '\0') {
							isOutputGem = TRUE;
						}
					}
				}
			} else if (found > 0 && uS.partcmp(utterance->speaker,kideval_CBS,FALSE,FALSE)) {
				if (kideval_n_option) {
				} else {
					if (isRightText(word)) {
						found--;
						if (found == 0) {
							if (kideval_SpecWords)
								isOutputGem = FALSE;
							else {
								isOutputGem = TRUE;
							}
						}
					}
				}
			}
			if (*utterance->speaker == '@' || isOutputGem) {
				if (utterance->speaker[0] == '*') {
					isPRCreateDBRes = TRUE;
					strcpy(templineC, utterance->speaker);
					ts = kideval_FindSpeaker(oldfname, templineC, NULL, TRUE, NULL);
				}
				if (command == BASE_COM) {
					kideval_process_tier(ts, NULL, NULL, IDText);
				} else if (command == DSS_COM) {
					if (dss_lang != '0') {
						lRightspeaker = kideval_process_dss_tier(ts, NULL, lRightspeaker, &lastdsp);
					}
				} else if (command == VOCD_COM) {
					kideval_process_vocd_tier(ts, NULL);
				} else if (command == IPSYN_COM) {
					if (strcmp(ipsyn_lang, IPSYNNOCOMPUTE) != 0) {
						if (utterance->speaker[0] == '*') {
							if (!isGRA) {
								fprintf(stderr,"\n*** File \"%s\": line %ld.\n", oldfname, lineno+tlineno);
								fprintf(stderr,"Speaker tier does not have corresponding %%GRA tier.\n");
								break;
							}
						} else if (fDepTierName[0] != EOS && uS.partcmp(utterance->speaker,fDepTierName,FALSE,FALSE)) {
							isGRA = FALSE;
						} else if (uS.partcmp(utterance->speaker,"%gra:",FALSE,FALSE)) {
							isGRA = TRUE;
						}
						lRightspeaker = kideval_process_ipsyn_tier(ts, NULL, lRightspeaker, &misalignmentError, &lastipsp);
					}
				}
			}
		}
	}
	if (isCreateDB == 0) {
		computeDSS_VOCD_IPSYN(misalignmentError);
	}
}

static struct label_cols *kideval_addNewLabel(char *label) {
	int  cnt;
	struct label_cols *p;

	cnt = 0;
	if (labelsRoot == NULL) {
		labelsRoot = NEW(struct label_cols);
		p = labelsRoot;
	} else {
		for (p=labelsRoot; p->next_label != NULL; p=p->next_label) {
			cnt++;
		}
		p->next_label = NEW(struct label_cols);
		p = p->next_label;
		cnt++;
	}
	if (p == NULL) {
		kideval_error(NULL, TRUE);
	}
	p->next_label = NULL;
	p->num = cnt;
	colLabelsNum++;
	if (label == NULL)
		p->label = NULL;
	else {
		templineC2[0] = EOS;
		if (!isRawVal) {
			strcpy(templineC2, "% ");
			strcat(templineC2, label);
		} else
			strcpy(templineC2, label);
		p->label = (char *)malloc(strlen(templineC2)+1);
		if (p->label == NULL) {
			kideval_error(NULL, TRUE);
		}
		strcpy(p->label, templineC2);
	}
	return(p);
}

static COLS *kideval_add2col(COLS *root, char isFirstElem, char *pat, const FNType *mFileName, int ln) {
	int  i;
	char ch, res;
	COLS *p;

	if (isFirstElem || root == NULL) {
		if (root == NULL) {
			root = NEW(COLS);
			p = root;
		} else {
			for (p=root; p->next_col != NULL; p=p->next_col) ;
			p->next_col = NEW(COLS);
			p = p->next_col;
		}
		if (p == NULL) {
			kideval_error(NULL, TRUE);
		}
		p->next_col = NULL;
		p->pats = NULL;
		p->labelP = NULL;
		p->isClitic = FALSE;
	} else
		for (p=root; p->next_col != NULL; p=p->next_col) ;
	if (pat[0] == '"' || pat[0] == '\'') {
		for (i=0; pat[i] != EOS && (pat[i] == '"' || pat[i] == '\'' || isSpace(pat[i])); i++) ;
		if (pat[i] != EOS) {
			if (p->labelP != NULL && p->labelP->label != NULL) {
				fprintf(stderr,"\n*** File \"%s\": line %d.\n", mFileName, ln);
				fprintf(stderr, "    ERROR: only one label is allow for each line: \"%s\"\n", pat);
				fprintf(stderr, "    Perhaps this label name is used on another line\n");
				kideval_error(gdb, FALSE);
			}
			p->labelP = kideval_addNewLabel(pat+i);
		}
	} else {
		i = 0;
		if (pat[i] == '-') {
			i++;
			ch = 'e';
		} else if (pat[i] == '+') {
			i++;
			ch = 'i';
		} else {
			ch = 0;
			fprintf(stderr,"\n*** File \"%s\": line %d.\n", mFileName, ln);
			fprintf(stderr, "    ERROR: Illegal character \"%c\" found on line \"%s\"\n", pat[i], pat);
			fprintf(stderr, "    Expected characters are: + - \" '\n\n");
			kideval_error(gdb, FALSE);
		}
		if (pat[i] == 's' || pat[i] == 'S')
			i++;
		if (pat[i] == '@')
			i++;
		if (strchr(pat+i, '~') != NULL || strchr(pat+i, '$') != NULL)
			p->isClitic = TRUE;
		p->pats = makeMorSeachList(p->pats, &res, pat+i, ch);
		if (p->pats == NULL) {
			if (res == TRUE)
				kideval_error(gdb, TRUE);
			else if (res == FALSE) {
				kideval_error(gdb, FALSE);
			}
		}
	}
	return(root);
}

static void processScriptLine(const FNType *mFileName, int  ln, const char *line) {
	char *b, *e, t, isQT, isFirstElem;

	if (line != NULL) // DO NOT REMOVE THIS LINE
		strcpy(templineC, line);
	if (uS.mStrnicmp(templineC, "dss ", 4) == 0) {
		b = templineC + 4;
		e = b;
		isQT = '\0';
		while (*b != EOS) {
			if (*e == '"' || *e == '\'') {
				if (isQT == *e)
					isQT = '\0';
				else if (isQT == '\0')
					isQT = *e;
			}
			if (((isSpace(*e) || *e == '"' || *e == '\'') && isQT == '\0') || *e == EOS) {
				t = *e;
				*e = EOS;
				strcpy(templineC1, b);
				uS.remblanks(templineC1);
				if ((templineC1[0] == '+' || templineC1[0] == '-') && (templineC1[1] == 'l' || templineC1[1] == 'L')) {
					if (templineC1[2] == 'e' || templineC1[2] == 'E') {
						dss_lang = 'e';
						if (rulesfile == NULL)
							rulesfile = DSSRULES;
					} else if (templineC1[2] == 'j' || templineC1[2] == 'J') {
						dss_lang = 'j';
						if (rulesfile == NULL)
							rulesfile = DSSJPRULES;
					} else if (templineC1[2] == '0') {
						dss_lang = '0';
						if (rulesfile == NULL)
							rulesfile = DSSNOCOMPUTE;
					} else {
						fprintf(stderr,"*** File \"%s\": line: \"%d\"\n", mFileName, ln);
						fprintf(stderr,"    Illegal option argument found: %s\n", templineC1);
						fprintf(stderr,"    Please specify either \"+le\" for English or \"+lj\" for Japanese.\n");
						cutt_exit(0);
					}
				} else if ((templineC1[0] == '+' || templineC1[0] == '-') && (templineC1[1] == 'd' || templineC1[1] == 'D')) {
					strncpy(dssRulesFName, templineC1+2, 255);
					dssRulesFName[255] = EOS;
					rulesfile = dssRulesFName;
					isDssFileSpecified = TRUE;
				} else {
					fprintf(stderr,"*** File \"%s\": line: \"%d\"\n", mFileName, ln);
					fprintf(stderr,"    Unsupported option specified: %s\n", templineC1);
					cutt_exit(0);
				}
				*e = t;
				if (*e != EOS)
					e++;
				b = e;
			} else
				e++;
		}
	} else if (uS.mStrnicmp(templineC, "ipsyn ", 6) == 0) {
		b = templineC + 6;
		e = b;
		isQT = '\0';
		while (*b != EOS) {
			if (*e == '"' || *e == '\'') {
				if (isQT == *e)
					isQT = '\0';
				else if (isQT == '\0')
					isQT = *e;
			}
			if (((isSpace(*e) || *e == '"' || *e == '\'') && isQT == '\0') || *e == EOS) {
				t = *e;
				*e = EOS;
				strcpy(templineC1, b);
				uS.remblanks(templineC1);
				if ((templineC1[0] == '+' || templineC1[0] == '-') && (templineC1[1] == 'l' || templineC1[1] == 'L')) {
					if (templineC1[2] == '0') {
						strcpy(ipsyn_lang, IPSYNNOCOMPUTE);
					} else {
						strncpy(ipsyn_lang, templineC1+2, 256);
						ipsyn_lang[256] = EOS;
					}
				} else {
					fprintf(stderr,"*** File \"%s\": line: \"%d\"\n", mFileName, ln);
					fprintf(stderr,"    Unsupported option specified: %s\n", templineC1);
					cutt_exit(0);
				}
				*e = t;
				if (*e != EOS)
					e++;
				b = e;
			} else
				e++;
		}
	} else {
		isFirstElem = TRUE;
		isQT = '\0';
		b = templineC;
		e = b;
		while (*b != EOS) {
			if (*e == '"' || *e == '\'') {
				if (isQT == *e)
					isQT = '\0';
				else if (isQT == '\0')
					isQT = *e;
			}
			if (((isSpace(*e) || *e == '"' || *e == '\'') && isQT == '\0') || *e == EOS) {
				t = *e;
				*e = EOS;
				strcpy(templineC1, b);
				uS.remblanks(templineC1);
				if (templineC1[0] != EOS) {
					colsRoot = kideval_add2col(colsRoot, isFirstElem, templineC1, mFileName, ln);
					isFirstElem = FALSE;
				}
				*e = t;
				if (*e != EOS)
					e++;
				b = e;
			} else
				e++;
		}
	}
}

static void processEng(void) {
	processScriptLine("Eng", 0, "dss +leng");
	processScriptLine("Eng", 0, "ipsyn +leng");
	processScriptLine("Eng", 0, "+-PRESP		\"*-PRESP\"");
	processScriptLine("Eng", 0, "+r-in		\"in\"");
	processScriptLine("Eng", 0, "+r-on		\"on\"");
	processScriptLine("Eng", 0, "+-PL		\"*-PL\"");
	processScriptLine("Eng", 0, "+&PAST		\"*&PAST\"");
	processScriptLine("Eng", 0, "+-POSS		\"*-POSS\"");
	processScriptLine("Eng", 0, "+|cop -|*,~|cop	\"cop|*\"");
	processScriptLine("Eng", 0, "+|det:art		\"det:art\"");
	processScriptLine("Eng", 0, "+-PAST		\"*-PAST\"");
	processScriptLine("Eng", 0, "+-3S		\"*-3S\"");
	processScriptLine("Eng", 0, "+&3S		\"*&3S\"");
	processScriptLine("Eng", 0, "+|aux -|*,~|aux	\"aux|*\"");
	processScriptLine("Eng", 0, "+|*,~|cop	\"~cop|*\"");
	processScriptLine("Eng", 0, "+|*,~|aux	\"~aux|*\"");
}

static void processFra(void) {
	processScriptLine("Fra", 0, "dss +l0");
	processScriptLine("Fra", 0, "ipsyn +l0");
	processScriptLine("Fra", 0, "+-PP +&PP	\"PP\"");
	processScriptLine("Fra", 0, "+-PRES +&PRES	\"PRES\"");
	processScriptLine("Fra", 0, "+-IMP +&IMP	\"IMP\"");
	processScriptLine("Fra", 0, "+-IMPF +&IMPF	\"IMPF\"");
	processScriptLine("Fra", 0, "+-FUT +&FUT	\"FUT\"");
	processScriptLine("Fra", 0, "+-COND +&COND	\"COND\"");
	processScriptLine("Fra", 0, "+-SUB +&SUB	\"SUB\"");
	processScriptLine("Fra", 0, "+-INF	\"INF\"");
	processScriptLine("Fra", 0, "+-PPRF	\"PPRF\"");
	processScriptLine("Fra", 0, "+&1s	\"1s\"");
	processScriptLine("Fra", 0, "+&2s	\"2s\"");
	processScriptLine("Fra", 0, "+&3s	\"3s\"");
	processScriptLine("Fra", 0, "+&13s	\"13s\"");
	processScriptLine("Fra", 0, "+&12s	\"12s\"");
	processScriptLine("Fra", 0, "+&m	\"m\"");
	processScriptLine("Fra", 0, "+&f	\"f\"");
	processScriptLine("Fra", 0, "+&pl +-PL	\"PL\"");
	processScriptLine("Fra", 0, "+&sg +-SG	\"SG\"");
	processScriptLine("Fra", 0, "+&f,&pl	\"f&pl\"");
	processScriptLine("Fra", 0, "+&m,&pl	\"m&pl\"");
	processScriptLine("Fra", 0, "+&f,&sg	\"f&sg\"");
	processScriptLine("Fra", 0, "+&m,&sg	\"m&sg\"");
	processScriptLine("Fra", 0, "+|pro	\"pro\"");
	processScriptLine("Fra", 0, "+|adv	\"adv\"");
	processScriptLine("Fra", 0, "+|adv:int	\"adv:int\"");
	processScriptLine("Fra", 0, "+|adv:place	\"adv:place\"");
	processScriptLine("Fra", 0, "+|co	\"co\"");
	processScriptLine("Fra", 0, "+|adj	\"adj\"");
	processScriptLine("Fra", 0, "+|n	\"n\"");
	processScriptLine("Fra", 0, "+|n:prop	\"n:prop\"");
	processScriptLine("Fra", 0, "+|num	\"num\"");
	processScriptLine("Fra", 0, "+|v	\"v\"");
	processScriptLine("Fra", 0, "+|v:mdllex	\"v:mdllex\"");
	processScriptLine("Fra", 0, "+|det:art	\"det:art\"");
	processScriptLine("Fra", 0, "+|det:dem	\"det:dem\"");
	processScriptLine("Fra", 0, "+|det:int	\"det:int\"");
	processScriptLine("Fra", 0, "+|det:gen	\"det:gen\"");
	processScriptLine("Fra", 0, "+|det:poss	\"det:poss\"");
	processScriptLine("Fra", 0, "+|prep	\"prep\"");
	processScriptLine("Fra", 0, "+|pro:ind	\"pro:ind\"");
	processScriptLine("Fra", 0, "+|pro:obj	\"pro:obj\"");
	processScriptLine("Fra", 0, "+|pro:refl	\"pro:refl\"");
	processScriptLine("Fra", 0, "+|pro:rel	\"pro:rel\"");
	processScriptLine("Fra", 0, "+|pro:subj	\"pro:subj\"");
	processScriptLine("Fra", 0, "+|pro:dem	\"pro:dem\"");
	processScriptLine("Fra", 0, "+|pro:int	\"pro:int\"");
	processScriptLine("Fra", 0, "+|pro:y	\"pro:y\"");
	processScriptLine("Fra", 0, "+|v:aux	\"v:aux\"");
	processScriptLine("Fra", 0, "+|v:mdl	\"v:mdl\"");
	processScriptLine("Fra", 0, "+|v:exist	\"v:exist\"");
	processScriptLine("Fra", 0, "+|v:poss	\"v:poss\"");
	processScriptLine("Fra", 0, "+|conj	\"conj\"");
	processScriptLine("Fra", 0, "+|adv:neg	\"adv:neg\"");
}

static void processSpa(void) {
	processScriptLine("Spa", 0, "dss +l0");
	processScriptLine("Spa", 0, "ipsyn +l0");
	processScriptLine("Spa", 0, "+-1S	\"*-1S\"");
	processScriptLine("Spa", 0, "+-2S	\"*-2S\"");
	processScriptLine("Spa", 0, "+-3S	\"*-3S\"");
	processScriptLine("Spa", 0, "+-1P	\"*-1P\"");
	processScriptLine("Spa", 0, "+-2P	\"*-2P\"");
	processScriptLine("Spa", 0, "+-3P	\"*-3P\"");
	processScriptLine("Spa", 0, "+-INF	\"*-INF\"");
	processScriptLine("Spa", 0, "+&IMP	\"*&IMP\"");
	processScriptLine("Spa", 0, "+-PROG	\"*-PROG\"");
	processScriptLine("Spa", 0, "+&PRES	\"*&PRES\"");
	processScriptLine("Spa", 0, "+&PRET	\"*&PRET\"");
	processScriptLine("Spa", 0, "+&FUT	\"*&FUT\"");
	processScriptLine("Spa", 0, "+&COND	\"*&COND\"");
	processScriptLine("Spa", 0, "+&SUB,&PRES	\"*&SUB&PRES\"");
	processScriptLine("Spa", 0, "+&SUB,&PAS	\"*&SUB&PAS\"");
	processScriptLine("Spa", 0, "+|pro:obj	\"|pro:obj\"");
	processScriptLine("Spa", 0, "+|pro:refl	\"|pro:refl\"");
	processScriptLine("Spa", 0, "+|pro:clit	\"|pro:clit\"");
}

static void processZho(void) {
	processScriptLine("Zho", 0, "dss +l0");
	processScriptLine("Zho", 0, "ipsyn +l0");
	processScriptLine("Zho", 0, "+|adj	\"adj\"");
	processScriptLine("Zho", 0, "+|cl	\"cl\"");
	processScriptLine("Zho", 0, "+|conn	\"conn\"");
	processScriptLine("Zho", 0, "+|det	\"det\"");
	processScriptLine("Zho", 0, "+|loc	\"loc\"");
	processScriptLine("Zho", 0, "+|n	\"n\"");
	processScriptLine("Zho", 0, "+|num	\"num\"");
	processScriptLine("Zho", 0, "+|pro	\"pro\"");
	processScriptLine("Zho", 0, "+|sfp	\"sfp\"");
	processScriptLine("Zho", 0, "+|v	\"v\"");
	processScriptLine("Zho", 0, "+|v:aux	\"v:aux\"");
	processScriptLine("Zho", 0, "+|wh	\"wh\"");
	processScriptLine("Zho", 0, "+|v:dirc	\"v:dirc\"");
	processScriptLine("Zho", 0, "+&DIM	\"DIM\"");
}

static void processJpn(void) {
	processScriptLine("Jpn", 0, "dss +ljpn");
	processScriptLine("Jpn", 0, "ipsyn +l0");
	processScriptLine("Jpn", 0, "+|n	\"n\"");
	processScriptLine("Jpn", 0, "+|n:mot	\"n:mot\"");
	processScriptLine("Jpn", 0, "+|n:vn	\"vn\"");
	processScriptLine("Jpn", 0, "+|n:vn:mot	\"vn:mot\"");
	processScriptLine("Jpn", 0, "+|v:v,|v:c,|v:ir	\"v\"");
	processScriptLine("Jpn", 0, "+|adj,|adj:mot	\"adj\"");
	processScriptLine("Jpn", 0, "+|adv	\"adv\"");
	processScriptLine("Jpn", 0, "+|v:*:sub	\"v:sub\"");
	processScriptLine("Jpn", 0, "+|v:cop	\"v:cop\"");
	processScriptLine("Jpn", 0, "+|ptl:snr	\"p-snrの\"");
	processScriptLine("Jpn", 0, "+|n*fml	\"fml\"");
	processScriptLine("Jpn", 0, "+|smod	\"smod\"");
	processScriptLine("Jpn", 0, "+|ptl:conj	\"p-conj\"");
	processScriptLine("Jpn", 0, "+|conj	\"conj\"");
	processScriptLine("Jpn", 0, "+|ptl:case	\"p-case\"");
	processScriptLine("Jpn", 0, "+|ptl:post	\"p-post\"");
	processScriptLine("Jpn", 0, "+|ptl:top	\"p-top\"");
	processScriptLine("Jpn", 0, "+|ptl:quot	\"p-quot\"");
	processScriptLine("Jpn", 0, "+|ptl:attr	\"p-attrの\"");
	processScriptLine("Jpn", 0, "+|ptl:foc	\"p-foc\"");
	processScriptLine("Jpn", 0, "+|ptl:fina	\"p-final\"");
	processScriptLine("Jpn", 0, "+|ptl:top,;wa	\"1後_p-topは\"");
	processScriptLine("Jpn", 0, "+|ptl:attr	\"1後_p-attrの\"");
	processScriptLine("Jpn", 0, "+-PRES	\"1後_V-PRESる\"");
	processScriptLine("Jpn", 0, "+-PAST	\"1後_V-PASTた\"");
	processScriptLine("Jpn", 0, "+-IMP:te	\"1後_V-IMP:te\"");
	processScriptLine("Jpn", 0, "+|ptl:case,;ga	\"2前_p-caseが\"");
	processScriptLine("Jpn", 0, "+|ptl:post,;de	\"2前_p-postで\"");
	processScriptLine("Jpn", 0, "+|ptl:post,;ni	\"2前_p-postに\"");
	processScriptLine("Jpn", 0, "+|ptl:post,;to	\"2前_p-postと\"");
	processScriptLine("Jpn", 0, "+|ptl:foc,;mo	\"2前_p-focも\"");
	processScriptLine("Jpn", 0, "+|ptl:quot,;tte	\"2前_p-quotって\"");
	processScriptLine("Jpn", 0, "+-COMPL	\"2前_COMPLちゃう\"");
	processScriptLine("Jpn", 0, "+-ASP	\"2前_ASPてる\"");
	processScriptLine("Jpn", 0, "+-NEG	\"2前_NEGない\"");
	processScriptLine("Jpn", 0, "+-POT	\"2後_POTれる\"");
	processScriptLine("Jpn", 0, "+-DESID	\"2後_DESIDたい\"");
	processScriptLine("Jpn", 0, "+-POL	\"2後_POLます\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;ku	\"2後_v:subてくる\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;ik	\"2後_v:subていく\"");
	processScriptLine("Jpn", 0, "+|ptl:case,;o	\"3前_p-caseを\"");
	processScriptLine("Jpn", 0, "+|ptl:post,;kara	\"3前_p-postから\"");
	processScriptLine("Jpn", 0, "+|ptl:post,;made	\"3前_p-postまで\"");
	processScriptLine("Jpn", 0, "+|ptl:foc,;dake	\"3前_p-focだけ\"");
	processScriptLine("Jpn", 0, "+|ptl:quot,;to	\"3前_p-quotと\"");
	processScriptLine("Jpn", 0, "+-CONN	\"3前_V-CONNて\"");
	processScriptLine("Jpn", 0, "+-HORT	\"3前_V-HORTよう\"");
	processScriptLine("Jpn", 0, "+-COND:tara	\"3後_V-CONDたら\"");
	processScriptLine("Jpn", 0, "+&GER	\"3後_V-GERい\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;age	\"3後_v:subてあげる\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;ar	\"3後_v:subてある\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;ok	\"3後_v:subておく\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;mi	\"3後_v:subてみる\"");
	processScriptLine("Jpn", 0, "+-COND:ba	\"4後_V-CONDば\"");
	processScriptLine("Jpn", 0, "+&OBL:*	\"4後_V-OBLなくちゃ\"");
	processScriptLine("Jpn", 0, "+-IMP:nasai	\"4後_V-IMPなさい\"");
	processScriptLine("Jpn", 0, "+-CAUS	\"4後_CAUSさせる\"");
	processScriptLine("Jpn", 0, "+-PASS	\"4後_PASSられる\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;kudasai	\"4後_v:subてください\"");
	processScriptLine("Jpn", 0, "+|v:*:sub,;moraw	\"4後_v:subてもらう\"");
}

static void read_script(char *lang) {
	int  ln;
	char *b;
	FNType mFileName[FNSize];
	FILE *fp;

	if (*lang == EOS) {
		fprintf(stderr,	"No language specified with +l option.\n");
		fprintf(stderr,"Please specify language script file name with \"+l\" option.\n");
		fprintf(stderr,"For example, \"kideval +leng\" or \"kideval +leng.cut\".\n");
		cutt_exit(0);
	}
	if (uS.mStricmp(lang, "eng") == 0) {
		fprintf(stderr,"    Using default, build into KIDEVAL source code, language file: %s\n", "eng");
		processEng();
		return;
	} else if (uS.mStricmp(lang, "fra") == 0) {
		fprintf(stderr,"    Using default, build into KIDEVAL source code, language file: %s\n", "fra");
		processFra();
		return;
	} else if (uS.mStricmp(lang, "spa") == 0) {
		fprintf(stderr,"    Using default, build into KIDEVAL source code, language file: %s\n", "spa");
		processSpa();
		return;
	} else if (uS.mStricmp(lang, "zho") == 0) {
		fprintf(stderr,"    Using default, build into KIDEVAL source code, language file: %s\n", "zho");
		processZho();
		return;
	} else if (uS.mStricmp(lang, "jpn") == 0) {
		fprintf(stderr,"    Using default, build into KIDEVAL source code, language file: %s\n", "jpn");
		processJpn();
		return;
	}
	strcpy(mFileName, lib_dir);
	addFilename2Path(mFileName, "kideval");
	addFilename2Path(mFileName, lang);
	if ((b=strchr(lang, '.')) != NULL) {
		if (uS.mStricmp(b, ".cut") != 0)
			strcat(mFileName, ".cut");
	} else
		strcat(mFileName, ".cut");
	if ((fp=fopen(mFileName,"r")) == NULL) {
		if (b != NULL) {
			if (uS.mStricmp(b, ".cut") == 0) {
				strcpy(templineC, wd_dir);
				addFilename2Path(templineC, lang);
				if ((fp=fopen(templineC,"r")) != NULL) {
					strcpy(mFileName, templineC);
				}
			}
		}
	}
	if (fp == NULL) {
		fprintf(stderr, "\nERROR: Can't locate language file: \"%s\".\n", mFileName);
		fprintf(stderr, "Check to see if \"lib\" directory in Commands window is set correctly.\n\n");
		cutt_exit(0);
	}
	fprintf(stderr,"    Using language file: %s\n", mFileName);
	ln = 0;
	while (fgets_cr(templineC, UTTLINELEN, fp)) {
		if (uS.isUTF8(templineC) || uS.partcmp(templineC, FONTHEADER, FALSE, FALSE) ||
			strncmp(templineC, SPECIALTEXTFILESTR, SPECIALTEXTFILESTRLEN) == 0)
			continue;
		ln++;
		if (templineC[0] == '%' || templineC[0] == '#')
			continue;
		uS.remFrontAndBackBlanks(templineC);
		if (templineC[0] != EOS)
			processScriptLine(mFileName, ln, NULL);
	}
	fclose(fp);
/*
	if (colsRoot == NULL) {
		fprintf(stderr,"Can't find any usable declarations in language script file \"%s\".\n", mFileName);
		cutt_exit(0);
	}
*/
/*
struct label_cols *p;
for (p=labelsRoot; p->next_label != NULL; p=p->next_label) {
	fprintf(stdout, "%s: %d\n", p->label, p->num);
}
*/
}

void init(char first) {
	int  i;
	struct tier *nt;
	FNType debugfile[FNSize];

	if (first) {
		ftime = TRUE;
		isLinkAge = FALSE;
		DB_version[0] = EOS;
		isRawVal = TRUE;
		script_file = NULL;
		stout = FALSE;
		outputOnlyData = TRUE;
		OverWriteFile = TRUE;
		FilterTier = 1;
		LocalTierSelect = TRUE;
		AddCEXExtension = ".xls";
		if (defheadtier != NULL) {
			if (defheadtier->nexttier != NULL)
				free(defheadtier->nexttier);
			free(defheadtier);
			defheadtier = NULL;
		}
		init_words(BASE_COM);
		dbfpout = NULL;
		isDBFilesList = FALSE;
		DBFilesListFP = NULL;
		GemMode = '\0';
		kideval_SpecWords = 0;
		kideval_group = FALSE;
		onlyApplyToDB = FALSE;
		isGOptionSet = FALSE;
		kideval_n_option = FALSE;
		isNOptionSet = FALSE;
		strcpy(fDepTierName, "%mor:");
		strcpy(kideval_BBS, "@*&#");
		strcpy(kideval_CBS, "@*&#");
		sp_head = NULL;
		colLabelsNum = 0;
		colsRoot = NULL;
		labelsRoot = NULL;
		gdb = NULL;
		DBKeyRoot = NULL;
		DBGemNum = 0;
		if (isCreateDB) {
			stout = TRUE;
			isRecursive = TRUE;
		}
		MinUttLimit = 0;
		DssIpsynUttLimit = 0;
		combinput = TRUE;
		gMisalignmentError = 0;
		if (!init_dss(first))
			kideval_error(NULL, TRUE);
		init_vocd(first);
		if (!init_ipsyn(first))
			kideval_error(NULL, FALSE);
	} else {
		if (ftime) {
			ftime = FALSE;
			if (script_file == NULL) {
				fprintf(stderr,"\nPlease specify language script file name with \"+l\" option.\n");
				fprintf(stderr,"For example, \"kideval +leng\" or \"kideval +leng.cut\".\n");
				cutt_exit(0);
			}
			if (DBKeyRoot != NULL && strcmp(script_file, "eng") && strcmp(script_file, "zho") && strcmp(script_file, "jpn")) {
				fprintf(stderr,"\nThe database can only be used with \"eng\", \"zho\" or \"jpn\" languages.\n");
				cutt_exit(0);
			}
			read_script(script_file);
			for (i=0; GlobalPunctuation[i]; ) {
				if (GlobalPunctuation[i] == '!' ||
					GlobalPunctuation[i] == '?' ||
					GlobalPunctuation[i] == '.') 
					strcpy(GlobalPunctuation+i,GlobalPunctuation+i+1);
				else
					i++;
			}
			if (!chatmode) {
				fputs("+/-t option is not allowed with TEXT format\n", stderr);
				kideval_error(NULL, FALSE);
			}
			if (DssIpsynUttLimit > 0 && DBKeyRoot != NULL) {
				fprintf(stderr,"\n+qN option can not be used with +d option, database.\n");
				cutt_exit(0);
			}
			if (isDBFilesList && DBKeyRoot == NULL) {
				fprintf(stderr,"\n+e1 option can only be used with +d option.\n");
				cutt_exit(0);
			}
			if (isCreateDB) {
				if (strcmp(script_file, "eng") && strcmp(script_file, "zho") && strcmp(script_file, "jpn")) {
					fprintf(stderr,"\nThe database can only be used with \"eng\", \"zho\" or \"jpn\" languages.\n");
					cutt_exit(0);
				}
			} else if (DBKeyRoot != NULL) {
				if (isDBFilesList && DBFilesListFP == NULL) {
					strcpy(debugfile, DATABSEFILESLIST);
					DBFilesListFP = fopen(debugfile, "w");
					if (DBFilesListFP == NULL) {
						fprintf(stderr, "Can't create file \"%s\", perhaps it is opened by another application\n", debugfile);
					}
#ifdef _MAC_CODE
					else
						settyp(debugfile, 'TEXT', the_file_creator.out, FALSE);
#endif
				}
				gdb = NEW(struct database);
				if (gdb == NULL)
					kideval_error(gdb, TRUE);
				gdb->morItems_sqr = (float *)malloc(sizeof(float) * colLabelsNum);
				if (gdb->morItems_sqr == NULL) {
					kideval_error(gdb, TRUE);
				}
				gdb->morItems = (float *)malloc(sizeof(float) * colLabelsNum);
				if (gdb->morItems == NULL) {
					kideval_error(gdb, TRUE);
				}
				init_db(gdb);
				ParseDatabase(gdb, script_file);
				CleanUpTempIDSpeakers();
				isRawVal = TRUE;
			}
			for (i=1; i < targc; i++) {
				if (*targv[i] == '+' || *targv[i] == '-') {
					if (targv[i][1] == 't') {
						if (targv[i][2] == '%')
							strcpy(fDepTierName, targv[i]+2);
						else
							maingetflag(targv[i], NULL, &i);
					}
				}
			}	
			for (nt=headtier; nt != NULL; nt=nt->nexttier) {
				if (nt->tcode[0] == '*')
					break;
			}
			if (nt == NULL) {
				fprintf(stderr, "\nPlease specify at least one speaker tier code with \"+t\" option.\n");
				cutt_exit(0);
			}
			if (!init_dss(first))
				kideval_error(NULL, FALSE);
			if (!init_ipsyn(first))
				kideval_error(NULL, FALSE);
		} else
			init_s_option(BASE_COM);
		init_vocd(first);
	}
}

static int addFilePath(const char *filePath, int argc, char *argv[]) {
	strcpy(templineC, wd_dir);
	addFilename2Path(templineC, filePath);
	argv[argc] = (char *)malloc(strlen(templineC)+1);
	if (argv[argc] == NULL) {
		kideval_error(NULL, TRUE);
	}
	strcpy(argv[argc], templineC);
	argc++;
	return(argc);
}
//UNIX start 16:17 end
static int fillUpEngFpFilePaths(int argc, char *argv[]) {
	argc = addFilePath("childes-data/Eng-NA/Bates/Free20/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Bates/Free28/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Bernstein/Children/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Bliss/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Bloom70/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Bloom73/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Braunwald/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Brown/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Clark/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Demetras1/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Demetras2/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Feldman/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Gathercole/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Gleason/Father/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Gleason/Mother/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Hall/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Higginson/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/HSLLD/HV1/TP/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/HSLLD/HV2/TP/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/HSLLD/HV3/TP/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/MacWhinney/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/McCune/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/NewEngland/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Post/*.cha", argc, argv);
	argc = addFilePath("phon-data/Eng-NA/Providence/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Sachs/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Snow/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Suppes/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Tardif/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Valian/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/VanHouten/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/VanKleeck/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Warren/*.cha", argc, argv);
	argc = addFilePath("childes-data/Eng-NA/Weist/*.cha", argc, argv);
	return(argc);
}

static int fillUpZhoMdFilePaths(int argc, char *argv[]) {
	argc = addFilePath("childes-data/Chinese/Mandarin/Tong/*.cha", argc, argv);
	argc = addFilePath("childes-data/Chinese/Mandarin/Zhou1/*.cha", argc, argv);
	return(argc);
}

static int fillUpJpnTdFilePaths(int argc, char *argv[]) {
	argc = addFilePath("childes-data/Japanese/Miyata/Aki/*.cha", argc, argv);
	argc = addFilePath("childes-data/Japanese/Miyata/Ryo/*.cha", argc, argv);
	argc = addFilePath("childes-data/Japanese/Miyata/Tai/*.cha", argc, argv);
	argc = addFilePath("childes-data/Japanese/Noji/*.cha", argc, argv);
	argc = addFilePath("childes-data/Japanese/Ogawa/*.cha", argc, argv);
	argc = addFilePath("childes-data/Japanese/Yokoyama/*.cha", argc, argv);
	return(argc);
}

CLAN_MAIN_RETURN main(int argc, char *argv[]) {
#ifdef UNX
	int len;
#endif
	int i, fileArgcStart, fileArgcEnd, langArgc, spArgc;
	time_t t;
	char *cOption, langPref[32], spPref[128];
	char isFileGiven, isLangGiven, isSpGiven;

#if defined(_MAC_CODE) || defined(_WIN32)
	targv = argv;
#endif
#ifdef UNX
	for (i=0; i < argc; i++) {
		targv[i] = argv[i];
	}
	argv = targv;
#endif
	isWinMode = IS_WIN_MODE;
	chatmode = CHAT_MODE;
	CLAN_PROG_NUM = KIDEVAL;
	OnlydataLimit = 0;
	UttlineEqUtterance = FALSE;
#ifdef DEBUGEVALRESULTS
	dbResults = NULL;
#endif
	cOption = NULL;
	DBKeyRoot = NULL;
	DBFilesListFP = NULL;
	isCreateDB = 0;
	isFileGiven = FALSE;
	isLangGiven = FALSE;
	isSpGiven = FALSE;
	for (i=1; i < argc; i++) {
		if (*argv[i] == '+' || *argv[i] == '-') {
			if (argv[i][1] == 'c') {
				isCreateDB = 1;
				cOption = argv[i]+2;
			} else if (argv[i][1] == 't') {
				if (argv[i][2] == '*')
					isSpGiven = TRUE;
				else if (argv[i][2] == '@' && argv[i][3] == 'I' && argv[i][4] == 'D' && argv[i][5] == '=')
					isSpGiven = TRUE;
			} else if (argv[i][1] == 'l') {
				isLangGiven = TRUE;
#ifdef UNX
			} else if (argv[i][1] == 'L') {
				strcpy(lib_dir, argv[i]+2);
				len = strlen(lib_dir);
				if (len > 0 && lib_dir[len-1] != '/')
					strcat(lib_dir, "/");
#endif
			}
		} else
			isFileGiven = TRUE;
	}
	fileArgcStart = 0;
	fileArgcEnd = 0;
	spArgc = 0;
	langArgc = 0;
	CRDBcnt = 0;
	DB_type[0] = EOS;
	wdOffset = strlen(wd_dir);
	if (wd_dir[wdOffset-1] != PATHDELIMCHR)
		wdOffset++;
	if (argc > 1) {
		if (!isSpGiven) {
			spArgc = argc;
			strcpy(spPref, "+t*CHI:");
			argv[spArgc] = spPref;
			argc++;
		}
		if (!isLangGiven) {
			langArgc = argc;
			strcpy(langPref, "+leng");
			argv[langArgc] = langPref;
			argc++;
		}
		if (isCreateDB) {
			if (!isLangGiven)
				strcpy(DB_type, "_fp");
			else {
				if (cOption == NULL) {
					fprintf(stderr, "\nUnknown database type specified with \"+c\" option.\n");
					cutt_exit(0);
				} else if (strcmp(cOption, "_fp") == 0 || strcmp(cOption, "_md") == 0 || strcmp(cOption, "_td") == 0) {
					strcpy(DB_type, cOption);
				} else {
					if (cOption != NULL) {
						fprintf(stderr, "\nUnknown database type specified with \"-c%s\" option.\n", cOption);
					} else
						fprintf(stderr, "\nUnknown database type specified with \"+c\" option.\n");
					cutt_exit(0);
				}
			}
			if (!isFileGiven) {
				fileArgcStart = argc;
				if (strcmp(DB_type, "_fp") == 0)
					argc = fillUpEngFpFilePaths(argc, argv);
				else if (strcmp(DB_type, "_md") == 0)
					argc = fillUpZhoMdFilePaths(argc, argv);
				else if (strcmp(DB_type, "_td") == 0)
					argc = fillUpJpnTdFilePaths(argc, argv);
				fileArgcEnd = argc;
			}
		}
	}
	targc = argc;
	debug_dss_fp = stdout;
#ifdef _CLAN_DEBUG
	t = 1283562804L; // RAND
#else
	t = time(NULL);
#endif
	srand(t);    // seed the random number generator
	VOCD_TYPE_D = FALSE;
	bmain(argc,argv,kideval_pr_result);

	if (DBKeyRoot != NULL) {
		printf("\n");
#ifdef UNX
		printf("CURRENT DATABASE COMPARISONS ARE PRELIMINARY AND SHOULD NOT BE USED FOR CLINICAL DIAGNOSIS.\n");
		printf("PLEASE SEE SLP MANUAL FOR RECOMMENDED INTERPRETATION.\n");
#else
		printf("%c%cCURRENT DATABASE COMPARISONS ARE PRELIMINARY AND SHOULD NOT BE USED FOR CLINICAL DIAGNOSIS.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
		printf("%c%cPLEASE SEE SLP MANUAL FOR RECOMMENDED INTERPRETATION.%c%c\n", ATTMARKER, error_start, ATTMARKER, error_end);
#endif
	}
	if (DBFilesListFP != NULL)
		fclose(DBFilesListFP);
	gdb = freedb(gdb);
	DBKeyRoot = freeDBKeys(DBKeyRoot);
	dss_cleanSearchMem();
	ipsyn_cleanSearchMem();
	labelsRoot = freeLabels(labelsRoot);
	colsRoot = freeColsP(colsRoot);
	if (gMisalignmentError != 0) {
		fprintf(stderr,"\nWARNING: FOUND %d %%mor: TIERS THAT DO NOT LINK IN SIZE TO THEIR SPEAKER TIERS.\n", gMisalignmentError);
		fprintf(stderr,"THIS WILL EFFECT THE RESULT OF IPSyn ANALYSES.\n");
	}
	if (isCreateDB) {
		fprintf(stderr,"\nDONE CREATING DATABASE.\n");
	}
	if (dbfpout != NULL) {
//		fprintf(stderr,"Output file <%s>\n",FileName1);
		fclose(dbfpout);
	}
	if (spArgc != 0) {
		argv[spArgc] = NULL;
		argc--;
	}
	if (langArgc != 0) {
		argv[langArgc] = NULL;
		argc--;
	}
	if (isCreateDB) {
		for (i=fileArgcStart; i < fileArgcEnd; i++) {
			if (argv[i] != NULL) {
				free(argv[i]);
				argv[i] = NULL;
			}
			argc--;
		}
	}
#ifdef DEBUGEVALRESULTS
	if (dbResults != NULL)
		fclose(dbResults);
#endif
}

void getflag(char *f, char *f1, int *i) {
	int j;
	char *morf, *t;

	f++;
	switch(*f++) {
		case 'a':
			MinUttLimit = atoi(f);
			break;
		case 'b':
			morf = getfarg(f,f1,i);
			if (*(f-2) == '-') {
				if (*morf != EOS) {
					for (j=0; rootmorf[j] != EOS; ) {
						if (uS.isCharInMorf(rootmorf[j],morf)) 
							strcpy(rootmorf+j,rootmorf+j+1);
						else
							j++;
					}
				} else
					rootmorf[0] = EOS;
			} else {
				if (*morf != EOS) {
					t = rootmorf;
					rootmorf = (char *)malloc(strlen(t)+strlen(morf)+1);
					if (rootmorf == NULL) {
						fprintf(stderr,"No more space left in core.\n");
						cutt_exit(1);
					}
					strcpy(rootmorf,t);
					strcat(rootmorf,morf);
					free(t);
				}
			}
			break;
		case 'c':
			break;
		case 'd':
			if (*f == EOS) {
				fprintf(stderr,"Missing argument for option: %s\n", f-2);
				cutt_exit(0);
			} else {
				t = strchr(f, '~');
				if (t != NULL) {
					*t = EOS;
					strcpy(DB_type, "_");
					strcat(DB_type, f);
					*t = '~';
					f = t + 1;
					if (*f == EOS) {
						fprintf(stderr,"Missing argument for option: %s\n", f-2);
						cutt_exit(0);
					}
				} else {
					fprintf(stderr,"Missing database type argument for option: %s\n", f-2);
					cutt_exit(0);
				}
				addDBKeys(f);
			}
			break;
		case 'e':
			if (*f == '1')
				isDBFilesList = TRUE;
			else {
				fprintf(stderr,"Invalid argument for option: %s\n", f-2);
				cutt_exit(0);
			}
			break;
		case 'g':
			if (*f == EOS) {
				if (*(f-2) == '+')
					kideval_group = TRUE;
				else {
					isGOptionSet = TRUE;
					onlyApplyToDB = TRUE;
				}
			} else {
				GemMode = 'i';
				kideval_SpecWords++;
				addDBGems(getfarg(f,f1,i));
			}
			break;
		case 'l':
			script_file = f;
			break;
		case 'L':
			if (strcmp(f-1, "LinkAge") == 0) {
				isLinkAge = TRUE;
			}
#ifdef UNX
			else {
				strcpy(lib_dir, f);
				j = strlen(lib_dir);
				if (j > 0 && lib_dir[j-1] != '/')
					strcat(lib_dir, "/");
			}
#endif
			break;
		case 'n':
			if (*(f-2) == '+') {
				kideval_n_option = TRUE;
				strcpy(kideval_BBS, "@G:");
				strcpy(kideval_CBS, "@*&#");
			} else {
				strcpy(kideval_BBS, "@BG:");
				strcpy(kideval_CBS, "@EG:");
			}
			isNOptionSet = TRUE;
			no_arg_option(f);
			break;
		case 'o':
			if (*f == '4') {
				isRawVal = FALSE;
			} else {
				fprintf(stderr,"Invalid argument for option: %s\n", f-2);
				cutt_exit(0);
			}
			break;
		case 'q':
			DssIpsynUttLimit = atoi(f);
			break;
		case 's':
			if (*f == '[') {
				if (*(f+1) == '-') {
					if (*(f-2) == '+')
						isLanguageExplicit = 2;
					maingetflag(f-2,f1,i);
				} else {
					fprintf(stderr, "Please specify only precodes \"[- ...]\" with +/-s option.");
					cutt_exit(0);
				}
			} else {
				fprintf(stderr,"Utterances containing \"%s\" are excluded by default.\n", f);
				fprintf(stderr,"Excluding \"%s\" is not allowed.\n", f);
				kideval_error(NULL, FALSE);
			}
			break;
		case 't':
			break;
		default:
			maingetflag(f-2,f1,i);
			break;
	}
}
